@@ -6,41 +6,34 @@
 *           assignment of expressions and lists to objects or lists
 */
 
-#include <stdlib.h>
-#include <string.h>
-#include <ctype.h>
-
-
-#include <kernel/mod2.h>
-
-#include <omalloc/omalloc.h>
+#include "kernel/mod2.h"
 
 #define TRANSEXT_PRIVATES
-#include <polys/ext_fields/transext.h>
+#include "polys/ext_fields/transext.h"
 
-#include <misc/options.h>
-#include <misc/intvec.h>
+#include "misc/options.h"
+#include "misc/intvec.h"
 
-#include <coeffs/coeffs.h>
-#include <coeffs/numbers.h>
-#include <coeffs/bigintmat.h>
+#include "coeffs/coeffs.h"
+#include "coeffs/numbers.h"
+#include "coeffs/bigintmat.h"
 
 
-#include <polys/ext_fields/algext.h>
+#include "polys/ext_fields/algext.h"
 
-#include <polys/monomials/ring.h>
-#include <polys/matpol.h>
-#include <polys/monomials/maps.h>
-#include <polys/nc/nc.h>
-#include <polys/nc/sca.h>
-#include <polys/prCopy.h>
+#include "polys/monomials/ring.h"
+#include "polys/matpol.h"
+#include "polys/monomials/maps.h"
+#include "polys/nc/nc.h"
+#include "polys/nc/sca.h"
+#include "polys/prCopy.h"
 
-#include <kernel/polys.h>
-#include <kernel/ideals.h>
-#include <kernel/GBEngine/kstd1.h>
-#include <kernel/oswrapper/timer.h>
-#include <kernel/combinatorics/stairc.h>
-#include <kernel/GBEngine/syz.h>
+#include "kernel/polys.h"
+#include "kernel/ideals.h"
+#include "kernel/GBEngine/kstd1.h"
+#include "kernel/oswrapper/timer.h"
+#include "kernel/combinatorics/stairc.h"
+#include "kernel/GBEngine/syz.h"
 
 //#include "weight.h"
 #include "tok.h"
@@ -113,23 +106,21 @@ static BOOLEAN jjSHORTOUT(leftv, leftv a)
   if (currRing != NULL)
   {
     BOOLEAN shortOut = (BOOLEAN)((long)a->Data());
-#if HAVE_CAN_SHORT_OUT
-    if (!shortOut)
+    if (shortOut==0)
       currRing->ShortOut = 0;
     else
     {
       if (currRing->CanShortOut)
         currRing->ShortOut = 1;
     }
-#else
-    currRing->ShortOut = shortOut;
+    shortOut = currRing->ShortOut;
     coeffs cf = currRing->cf;
-    while (nCoeff_is_Extension(cf)) {
+    while (nCoeff_is_Extension(cf))
+    {
       cf->extRing->ShortOut = shortOut;
       assume(cf->extRing != NULL);
       cf = cf->extRing->cf;
     }
-#endif
   }
   return FALSE;
 }
@@ -172,12 +163,95 @@ static void jjMINPOLY_red(idhdl h)
       {
         jjMINPOLY_red((idhdl)&(L->m[i]));
       }
+      break;
     }
     default:
     //case RESOLUTION_CMD:
        Werror("type %d too complex...set minpoly before",IDTYP(h)); break;
   }
 }
+ring jjSetMinpoly(ring r, number a)
+{
+  if ( !nCoeff_is_transExt(r->cf) )
+  {
+    if(!nCoeff_is_algExt(r->cf) )
+    {
+      WerrorS("cannot set minpoly for these coeffients");
+      return NULL;
+    }
+  }
+  if (rVar(r->cf->extRing)!=1)
+  {
+    WerrorS("only univarite minpoly allowed");
+    return NULL;
+  }
+
+  assume (r->idroot==NULL);
+
+  number p = n_Copy(a,r->cf);
+  n_Normalize(p, r->cf);
+
+  if (n_IsZero(p, r->cf))
+  {
+    n_Delete(&p, r->cf);
+    if( nCoeff_is_transExt(r->cf) )
+    {
+      return r;
+    }
+    WarnS("cannot set minpoly to 0 / alg. extension?");
+    return NULL;
+  }
+  r=rCopy(r);
+  // remove all object currently in the ring
+
+  AlgExtInfo A;
+
+  A.r = rCopy(r->cf->extRing); // Copy  ground field!
+  // if minpoly was already set:
+  if( r->cf->extRing->qideal != NULL ) id_Delete(&(A.r->qideal),A.r);
+  ideal q = idInit(1,1);
+  if ((p==NULL) ||(NUM((fraction)p)==NULL))
+  {
+    WerrorS("Could not construct the alg. extension: minpoly==0");
+    // cleanup A: TODO
+    rDelete( A.r );
+    rDelete(r);
+    return NULL;
+  }
+  if (DEN((fraction)(p)) != NULL) // minpoly must be a fraction with poly numerator...!!
+  {
+    poly n=DEN((fraction)(p));
+    if(!p_IsConstant(n,r->cf->extRing))
+    {
+      WarnS("denominator must be constant - ignoring it");
+    }
+    p_Delete(&n,r->cf->extRing);
+    DEN((fraction)(p))=NULL;
+  }
+
+  q->m[0] = NUM((fraction)p);
+  A.r->qideal = q;
+
+  EXTERN_VAR omBin fractionObjectBin;
+  NUM((fractionObject *)p) = NULL; // not necessary, but still...
+  omFreeBin((ADDRESS)p, fractionObjectBin);
+
+  coeffs new_cf = nInitChar(n_algExt, &A);
+  if (new_cf==NULL)
+  {
+    WerrorS("Could not construct the alg. extension: llegal minpoly?");
+    // cleanup A: TODO
+    rDelete( A.r );
+    rDelete(r);
+    return NULL;
+  }
+  else
+  {
+    nKillChar(r->cf); r->cf=new_cf;
+  }
+  return r;
+}
+
 static BOOLEAN jjMINPOLY(leftv, leftv a)
 {
   if( !nCoeff_is_transExt(currRing->cf) && (currRing->idroot == NULL) && n_IsZero((number)a->Data(), currRing->cf) )
@@ -206,8 +280,10 @@ static BOOLEAN jjMINPOLY(leftv, leftv a)
     return TRUE;
   }
 
+  BOOLEAN redefine_from_algext=FALSE;
   if ( currRing->idroot != NULL )
   {
+    redefine_from_algext=(currRing->cf->extRing->qideal!=NULL);
 //    return TRUE;
 #ifndef SING_NDEBUG
     idhdl p = currRing->idroot;
@@ -216,7 +292,7 @@ static BOOLEAN jjMINPOLY(leftv, leftv a)
 
     while(p != NULL)
     {
-      PrintS(p->String(TRUE)); PrintLn();
+      PrintS(p->String(TRUE)); Print("(%s)\n",IDID(p));
       p = p->next;
     }
 #endif
@@ -263,17 +339,19 @@ static BOOLEAN jjMINPOLY(leftv, leftv a)
     rDelete( A.r );
     return TRUE;
   }
-  if (DEN((fraction)(p)) != NULL) // minpoly must be a fraction with poly numerator...!!
+  if (!redefine_from_algext && (DEN((fraction)(p)) != NULL)) // minpoly must be a fraction with poly numerator...!!
   {
-    poly z=NUM((fraction)p);
     poly n=DEN((fraction)(p));
-    z=p_Mult_nn(z,pGetCoeff(n),currRing->cf->extRing);
-    NUM((fraction)p)=z;
-    DEN((fraction)(p))=NULL;
+    if(!p_IsConstant(n,currRing->cf->extRing))
+    {
+      WarnS("denominator must be constant - ignoring it");
+    }
     p_Delete(&n,currRing->cf->extRing);
+    DEN((fraction)(p))=NULL;
   }
 
-  q->m[0] = NUM((fraction)p);
+  if (redefine_from_algext) q->m[0]=(poly)p;
+  else          q->m[0] = NUM((fraction)p);
   A.r->qideal = q;
 
 #if 0
@@ -287,16 +365,14 @@ static BOOLEAN jjMINPOLY(leftv, leftv a)
   // :(
 //  NUM((fractionObject *)p) = NULL; // makes 0/ NULL fraction - which should not happen!
 //  n_Delete(&p, currRing->cf); // doesn't expect 0/ NULL :(
-  if(true)
+  if (!redefine_from_algext)
   {
-    extern omBin fractionObjectBin;
+    EXTERN_VAR omBin fractionObjectBin;
     NUM((fractionObject *)p) = NULL; // not necessary, but still...
     omFreeBin((ADDRESS)p, fractionObjectBin);
   }
 
-
   coeffs new_cf = nInitChar(n_algExt, &A);
-
   if (new_cf==NULL)
   {
     WerrorS("Could not construct the alg. extension: llegal minpoly?");
@@ -308,9 +384,10 @@ static BOOLEAN jjMINPOLY(leftv, leftv a)
   {
     nKillChar(currRing->cf); currRing->cf=new_cf;
   }
-
   return FALSE;
 }
+
+
 static BOOLEAN jjNOETHER(leftv, leftv a)
 {
   poly p=(poly)a->CopyD(POLY_CMD);
@@ -398,6 +475,13 @@ static BOOLEAN jiA_INT(leftv res, leftv a, Subexpr e)
 }
 static BOOLEAN jiA_NUMBER(leftv res, leftv a, Subexpr)
 {
+  void *test_p=a->Data(); // can I access it (newstruct)?
+  if (errorreported) return TRUE;
+  if (Sy_inset(FLAG_OTHER_RING,res->flag))
+  {
+    (res-1)->data=currRing;
+    (res-1)->rtyp=RING_CMD;
+  }
   number p=(number)a->CopyD(NUMBER_CMD);
   if (res->data!=NULL) nDelete((number *)&res->data);
   nNormalize(p);
@@ -622,6 +706,13 @@ static BOOLEAN jiA_BIGINT(leftv res, leftv a, Subexpr e)
 }
 static BOOLEAN jiA_LIST_RES(leftv res, leftv a,Subexpr)
 {
+  void *test_p=a->Data(); // can I access it (newstruct)?
+  if (errorreported) return TRUE;
+  if (Sy_inset(FLAG_OTHER_RING,res->flag))
+  {
+    (res-1)->data=currRing;
+    (res-1)->rtyp=RING_CMD;
+  }
   syStrategy r=(syStrategy)a->CopyD(RESOLUTION_CMD);
   if (res->data!=NULL) ((lists)res->data)->Clean();
   int add_row_shift = 0;
@@ -633,6 +724,13 @@ static BOOLEAN jiA_LIST_RES(leftv res, leftv a,Subexpr)
 }
 static BOOLEAN jiA_LIST(leftv res, leftv a,Subexpr)
 {
+  void *test_p=a->Data(); // can I access it (newstruct)?
+  if (errorreported) return TRUE;
+  if (Sy_inset(FLAG_OTHER_RING,res->flag))
+  {
+    (res-1)->data=currRing;
+    (res-1)->rtyp=RING_CMD;
+  }
   lists l=(lists)a->CopyD(LIST_CMD);
   if (res->data!=NULL) ((lists)res->data)->Clean();
   res->data=(void *)l;
@@ -641,6 +739,13 @@ static BOOLEAN jiA_LIST(leftv res, leftv a,Subexpr)
 }
 static BOOLEAN jiA_POLY(leftv res, leftv a,Subexpr e)
 {
+  void *test_p=a->Data(); // can I access it (newstruct)?
+  if (errorreported) return TRUE;
+  if (Sy_inset(FLAG_OTHER_RING,res->flag))
+  {
+    (res-1)->data=currRing;
+    (res-1)->rtyp=RING_CMD;
+  }
   poly p=(poly)a->CopyD(POLY_CMD);
   pNormalize(p);
   if (e==NULL)
@@ -648,7 +753,7 @@ static BOOLEAN jiA_POLY(leftv res, leftv a,Subexpr e)
     if ((p!=NULL) && TEST_V_QRING && (currRing->qideal!=NULL)
     && (!hasFlag(a,FLAG_QRING)))
     {
-      jjNormalizeQRingP(p);
+      p=jjNormalizeQRingP(p);
       setFlag(res,FLAG_QRING);
     }
     if (res->data!=NULL) pDelete((poly*)&res->data);
@@ -668,7 +773,7 @@ static BOOLEAN jiA_POLY(leftv res, leftv a,Subexpr e)
       {
         if (TEST_V_ALLWARN)
         {
-          Warn("increase ideal %d -> %d in %s",MATCOLS(m),j,my_yylinebuf);
+          Warn("increase ideal %d -> %d in %s(%d):%s",MATCOLS(m),j,VoiceName(),VoiceLine(),my_yylinebuf);
         }
         pEnlargeSet(&(m->m),MATCOLS(m),j-MATCOLS(m));
         MATCOLS(m)=j;
@@ -686,14 +791,23 @@ static BOOLEAN jiA_POLY(leftv res, leftv a,Subexpr e)
     }
     if ((p!=NULL) && TEST_V_QRING && (currRing->qideal!=NULL))
     {
-      jjNormalizeQRingP(p);
+      p=jjNormalizeQRingP(p);
     }
-    pDelete(&MATELEM(m,i,j));
-    MATELEM(m,i,j)=p;
-    /* for module: update rank */
-    if ((p!=NULL) && (pGetComp(p)!=0))
+    if (res->rtyp==SMATRIX_CMD)
     {
-      m->rank=si_max(m->rank,pMaxComp(p));
+      p=pSub(p,SMATELEM(m,i-1,j-1,currRing));
+      pSetCompP(p,i);
+      m->m[j-1]=pAdd(m->m[j-1],p);
+    }
+    else
+    {
+      pDelete(&MATELEM(m,i,j));
+      MATELEM(m,i,j)=p;
+      /* for module: update rank */
+      if ((p!=NULL) && (pGetComp(p)!=0))
+      {
+        m->rank=si_max(m->rank,pMaxComp(p));
+      }
     }
   }
   return FALSE;
@@ -815,8 +929,34 @@ static BOOLEAN jiA_BIGINTMAT(leftv res, leftv a, Subexpr)
   jiAssignAttr(res,a);
   return FALSE;
 }
+static BOOLEAN jiA_BUCKET(leftv res, leftv a, Subexpr e)
+// there should be no assign bucket:=bucket, here we have poly:=bucket
+{
+  if (Sy_inset(FLAG_OTHER_RING,res->flag))
+  {
+    (res-1)->data=currRing;
+    (res-1)->rtyp=RING_CMD;
+  }
+  void *test_p=a->Data(); // can I access it (newstruct)?
+  if (errorreported) return TRUE;
+  sBucket_pt b=(sBucket_pt)a->CopyD();
+  poly p; int l;
+  sBucketDestroyAdd(b,&p,&l);
+  sleftv tmp;
+  tmp.Init();
+  tmp.rtyp=POLY_CMD;
+  tmp.data=p;
+  return jiA_POLY(res,&tmp,e);
+}
 static BOOLEAN jiA_IDEAL(leftv res, leftv a, Subexpr)
 {
+  if (Sy_inset(FLAG_OTHER_RING,res->flag))
+  {
+    (res-1)->data=currRing;
+    (res-1)->rtyp=RING_CMD;
+  }
+  void *test_p=a->Data(); // can I access it (newstruct)?
+  if (errorreported) return TRUE;
   if (res->data!=NULL) idDelete((ideal*)&res->data);
   res->data=(void *)a->CopyD(MATRIX_CMD);
   if (a->rtyp==IDHDL) id_Normalize((ideal)a->Data(), currRing);
@@ -830,11 +970,22 @@ static BOOLEAN jiA_IDEAL(leftv res, leftv a, Subexpr)
   {
     setFlag(res,FLAG_STD);
   }
-  if (TEST_V_QRING && (currRing->qideal!=NULL)&& (!hasFlag(res,FLAG_QRING))) jjNormalizeQRingId(res);
+  if (TEST_V_QRING && (currRing->qideal!=NULL))
+  {
+    if (hasFlag(a,FLAG_QRING)) setFlag(res,FLAG_QRING);
+    else                       jjNormalizeQRingId(res);
+  }
   return FALSE;
 }
 static BOOLEAN jiA_RESOLUTION(leftv res, leftv a, Subexpr)
 {
+  void *test_p=a->Data(); // can I access it (newstruct)?
+  if (Sy_inset(FLAG_OTHER_RING,res->flag))
+  {
+    (res-1)->data=currRing;
+    (res-1)->rtyp=RING_CMD;
+  }
+  if (errorreported) return TRUE;
   if (res->data!=NULL) syKillComputation((syStrategy)res->data);
   res->data=(void *)a->CopyD(RESOLUTION_CMD);
   jiAssignAttr(res,a);
@@ -843,6 +994,13 @@ static BOOLEAN jiA_RESOLUTION(leftv res, leftv a, Subexpr)
 static BOOLEAN jiA_MODUL_P(leftv res, leftv a, Subexpr)
 /* module = poly */
 {
+  void *test_p=a->Data(); // can I access it (newstruct)?
+  if (Sy_inset(FLAG_OTHER_RING,res->flag))
+  {
+    (res-1)->data=currRing;
+    (res-1)->rtyp=RING_CMD;
+  }
+  if (errorreported) return TRUE;
   if (res->data!=NULL) idDelete((ideal*)&res->data);
   ideal I=idInit(1,1);
   I->m[0]=(poly)a->CopyD(POLY_CMD);
@@ -858,6 +1016,13 @@ static BOOLEAN jiA_MODUL_P(leftv res, leftv a, Subexpr)
 }
 static BOOLEAN jiA_IDEAL_M(leftv res, leftv a, Subexpr)
 {
+  void *test_p=a->Data(); // can I access it (newstruct)?
+  if (errorreported) return TRUE;
+  if (Sy_inset(FLAG_OTHER_RING,res->flag))
+  {
+    (res-1)->data=currRing;
+    (res-1)->rtyp=RING_CMD;
+  }
   if (res->data!=NULL) idDelete((ideal*)&res->data);
   matrix m=(matrix)a->CopyD(MATRIX_CMD);
   if (TEST_V_ALLWARN)
@@ -868,7 +1033,38 @@ static BOOLEAN jiA_IDEAL_M(leftv res, leftv a, Subexpr)
   MATROWS(m)=1;
   id_Normalize((ideal)m, currRing);
   res->data=(void *)m;
-  if (TEST_V_QRING && (currRing->qideal!=NULL)) jjNormalizeQRingId(res);
+  if (TEST_V_QRING && (currRing->qideal!=NULL))
+  {
+    if (hasFlag(a,FLAG_QRING)) setFlag(res,FLAG_QRING);
+    else                       jjNormalizeQRingId(res);
+  }
+  return FALSE;
+}
+static BOOLEAN jiA_IDEAL_Mo(leftv res, leftv a, Subexpr)
+{
+  void *test_p=a->Data(); // can I access it (newstruct)?
+  if (errorreported) return TRUE;
+  if (Sy_inset(FLAG_OTHER_RING,res->flag))
+  {
+    (res-1)->data=currRing;
+    (res-1)->rtyp=RING_CMD;
+  }
+  ideal m=(ideal)a->CopyD(MODUL_CMD);
+  if (m->rank>1)
+  {
+    Werror("rank of module is %ld in assignment to ideal",m->rank);
+    return TRUE;
+  }
+  if (res->data!=NULL) idDelete((ideal*)&res->data);
+  id_Normalize(m, currRing);
+  id_Shift(m,-1,currRing);
+  m->rank=1;
+  res->data=(void *)m;
+  if (TEST_V_QRING && (currRing->qideal!=NULL))
+  {
+    if (hasFlag(a,FLAG_QRING)) setFlag(res,FLAG_QRING);
+    else                       jjNormalizeQRingId(res);
+  }
   return FALSE;
 }
 static BOOLEAN jiA_LINK(leftv res, leftv a, Subexpr)
@@ -897,6 +1093,13 @@ static BOOLEAN jiA_LINK(leftv res, leftv a, Subexpr)
 // assign map -> map
 static BOOLEAN jiA_MAP(leftv res, leftv a, Subexpr)
 {
+  void *test_p=a->Data(); // can I access it (newstruct)?
+  if (errorreported) return TRUE;
+  if (Sy_inset(FLAG_OTHER_RING,res->flag))
+  {
+    (res-1)->data=currRing;
+    (res-1)->rtyp=RING_CMD;
+  }
   if (res->data!=NULL)
   {
     omFree((ADDRESS)((map)res->data)->preimage);
@@ -910,6 +1113,13 @@ static BOOLEAN jiA_MAP(leftv res, leftv a, Subexpr)
 // assign ideal -> map
 static BOOLEAN jiA_MAP_ID(leftv res, leftv a, Subexpr)
 {
+  void *test_p=a->Data(); // can I access it (newstruct)?
+  if (errorreported) return TRUE;
+  if (Sy_inset(FLAG_OTHER_RING,res->flag))
+  {
+    (res-1)->data=currRing;
+    (res-1)->rtyp=RING_CMD;
+  }
   map f=(map)res->data;
   char *rn=f->preimage; // save the old/already assigned preimage ring name
   f->preimage=NULL;
@@ -929,6 +1139,14 @@ static BOOLEAN jiA_QRING(leftv res, leftv a,Subexpr e)
     WerrorS("qring_id expected");
     return TRUE;
   }
+  void *test_p=a->Data(); // can I access it (newstruct)?
+  if (errorreported) return TRUE;
+  if (Sy_inset(FLAG_OTHER_RING,res->flag))
+  {
+    (res-1)->data=currRing;
+    (res-1)->rtyp=RING_CMD;
+  }
+
   ring old_ring=(ring)res->Data();
 
   coeffs newcf = currRing->cf;
@@ -1083,9 +1301,8 @@ static BOOLEAN jiA_CRING(leftv res, leftv a, Subexpr)
 /*2
 * assign a = b
 */
-static BOOLEAN jiAssign_1(leftv l, leftv r, BOOLEAN toplevel)
+static BOOLEAN jiAssign_1(leftv l, leftv r, int rt, BOOLEAN toplevel, BOOLEAN is_qring=FALSE)
 {
-  int rt=r->Typ();
   if (rt==0)
   {
     if (!errorreported) Werror("`%s` is undefined",r->Fullname());
@@ -1100,16 +1317,20 @@ static BOOLEAN jiAssign_1(leftv l, leftv r, BOOLEAN toplevel)
   }
   if(rt==NONE)
   {
-    WarnS("right side is not a datum, assignment ignored");
-    Warn("in line >>%s<<",my_yylinebuf);
-    // if (!errorreported)
-    //   WerrorS("right side is not a datum");
-    //return TRUE;
+    if ((!TEST_V_ASSIGN_NONE)||(lt!=DEF_CMD))
+    {
+      WarnS("right side is not a datum, assignment ignored");
+      Warn("in line >>%s<<",my_yylinebuf);
+      // if (!errorreported)
+      //   WerrorS("right side is not a datum");
+      //return TRUE;
+    }
     return FALSE;
   }
 
   if (lt==DEF_CMD)
   {
+
     if (TEST_V_ALLWARN
     && (rt!=RING_CMD)
     && (l->name!=NULL)
@@ -1122,19 +1343,29 @@ static BOOLEAN jiAssign_1(leftv l, leftv r, BOOLEAN toplevel)
     }
     if (l->rtyp==IDHDL)
     {
-      IDTYP((idhdl)l->data)=rt;
+      if((currRingHdl==NULL) && RingDependend(rt))
+      {
+        WerrorS("basering required");
+        return TRUE;
+      }
+      if (rt==BUCKET_CMD) IDTYP((idhdl)l->data)=POLY_CMD;
+      else                IDTYP((idhdl)l->data)=rt;
     }
     else if (l->name!=NULL)
     {
+      int rrt;
+      if (rt==BUCKET_CMD) rrt=POLY_CMD;
+      else                rrt=rt;
       sleftv ll;
-      iiDeclCommand(&ll,l,myynest,rt,&IDROOT);
+      iiDeclCommand(&ll,l,myynest,rrt,&IDROOT);
       memcpy(l,&ll,sizeof(sleftv));
     }
     else
     {
-      l->rtyp=rt;
+      if (rt==BUCKET_CMD) l->rtyp=POLY_CMD;
+      else                l->rtyp=rt;
     }
-    lt=rt;
+    lt=l->Typ();
   }
   else
   {
@@ -1160,6 +1391,12 @@ static BOOLEAN jiAssign_1(leftv l, leftv r, BOOLEAN toplevel)
 #endif
     return (bb==NULL) || bb->blackbox_Assign(l,r);
   }
+  if ((is_qring)
+  &&(lt==RING_CMD)
+  &&(rt==RING_CMD))
+  {
+    Warn("qring .. = <ring>; is misleading in >>%s<<",my_yylinebuf);
+  }
   int start=0;
   while ((dAssign[start].res!=lt)
       && (dAssign[start].res!=0)) start++;
@@ -1337,7 +1574,7 @@ static BOOLEAN jiA_INTVEC_L(leftv l,leftv r)
     t.data=(char *)(long)(*iv)[i];
     h=l->next;
     l->next=NULL;
-    nok=jiAssign_1(l,&t,TRUE);
+    nok=jiAssign_1(l,&t,INT_CMD,TRUE);
     l->next=h;
     if (nok) return TRUE;
     i++;
@@ -1356,9 +1593,9 @@ static BOOLEAN jiA_VECTOR_L(leftv l,leftv r)
   leftv h;
   sleftv t;
   int i=0;
+  memset(&t,0,sizeof(sleftv));
   while (l!=NULL)
   {
-    memset(&t,0,sizeof(sleftv));
     t.rtyp=POLY_CMD;
     if (i>=IDELEMS(I))
     {
@@ -1371,7 +1608,7 @@ static BOOLEAN jiA_VECTOR_L(leftv l,leftv r)
     }
     h=l->next;
     l->next=NULL;
-    nok=jiAssign_1(l,&t,TRUE);
+    nok=jiAssign_1(l,&t,POLY_CMD,TRUE);
     l->next=h;
     t.CleanUp();
     if (nok)
@@ -1385,7 +1622,7 @@ static BOOLEAN jiA_VECTOR_L(leftv l,leftv r)
   idDelete(&I);
   l1->CleanUp();
   r->CleanUp();
-  //if (TEST_V_QRING && (currRing->qideal!=NULL)) jjNormalizeQRingP(l);
+  //if (TEST_V_QRING && (currRing->qideal!=NULL)) l=jjNormalizeQRingP(l);
   return FALSE;
 }
 static BOOLEAN jjA_L_LIST(leftv l, leftv r)
@@ -1609,7 +1846,7 @@ static BOOLEAN jiA_MATRIX_L(leftv l,leftv r)
       l->next=NULL;
       idhdl hh=NULL;
       if ((l->rtyp==IDHDL)&&(l->Typ()==DEF_CMD)) hh=(idhdl)l->data;
-      nok=jiAssign_1(l,&t,TRUE);
+      nok=jiAssign_1(l,&t,POLY_CMD,TRUE);
       if (hh!=NULL) { ipMoveId(hh);hh=NULL;}
       l->next=h;
       if (nok)
@@ -1627,14 +1864,14 @@ static BOOLEAN jiA_MATRIX_L(leftv l,leftv r)
     {
       if (r!=NULL)
       {
-        Warn("list length mismatch in assign (l>r)");
+        WarnS("list length mismatch in assign (l>r)");
         nok=TRUE;
       }
       break;
     }
     else if (r==NULL)
     {
-      Warn("list length mismatch in assign (l<r)");
+      WarnS("list length mismatch in assign (l<r)");
       nok=TRUE;
       break;
     }
@@ -1685,7 +1922,7 @@ static BOOLEAN jiA_STRING_L(leftv l,leftv r)
     t.data=ss;
     h=l->next;
     l->next=NULL;
-    nok=jiAssign_1(l,&t,TRUE);
+    nok=jiAssign_1(l,&t,STRING_CMD,TRUE);
     if (nok)
     {
       break;
@@ -1724,7 +1961,7 @@ static BOOLEAN jiAssign_list(leftv l, leftv r)
   {
     if (TEST_V_ALLWARN)
     {
-      Warn("increase list %d -> %d in %s",li->nr,i,my_yylinebuf);
+      Warn("increase list %d -> %d in %s(%d):%s",li->nr,i,VoiceName(),VoiceLine(),my_yylinebuf);
     }
     li->m=(leftv)omreallocSize(li->m,(li->nr+1)*sizeof(sleftv),(i+1)*sizeof(sleftv));
     memset(&(li->m[li->nr+1]),0,(i-li->nr)*sizeof(sleftv));
@@ -1736,12 +1973,12 @@ static BOOLEAN jiAssign_list(leftv l, leftv r)
   leftv ld=&(li->m[i]);
   ld->e=l->e->next;
   BOOLEAN b;
+  sleftv tmp;
+  memset(&tmp,0,sizeof(sleftv));
   if (/*(ld->rtyp!=LIST_CMD)
   &&*/(ld->e==NULL)
   && (ld->Typ()!=r->Typ()))
   {
-    sleftv tmp;
-    memset(&tmp,0,sizeof(sleftv));
     tmp.rtyp=DEF_CMD;
     b=iiAssign(&tmp,r,FALSE);
     ld->CleanUp();
@@ -1751,8 +1988,6 @@ static BOOLEAN jiAssign_list(leftv l, leftv r)
   && (ld->Typ()==r->Typ())
   && (ld->Typ()<MAX_TOK))
   {
-    sleftv tmp;
-    memset(&tmp,0,sizeof(sleftv));
     tmp.rtyp=r->Typ();
     tmp.data=(char*)idrecDataInit(r->Typ());
     b=iiAssign(&tmp,r,FALSE);
@@ -1797,7 +2032,8 @@ BOOLEAN iiAssign(leftv l, leftv r, BOOLEAN toplevel)
   int rl;
   int lt=l->Typ();
   int rt=NONE;
-  BOOLEAN b;
+  int is_qring=FALSE;
+  BOOLEAN b=FALSE;
   if (l->rtyp==ALIAS_CMD)
   {
     Werror("`%s` is read-only",l->Name());
@@ -1806,6 +2042,7 @@ BOOLEAN iiAssign(leftv l, leftv r, BOOLEAN toplevel)
   if (l->rtyp==IDHDL)
   {
     atKillAll((idhdl)l->data);
+    is_qring=hasFlag((idhdl)l->data,FLAG_QRING_DEF);
     IDFLAG((idhdl)l->data)=0;
     l->attribute=NULL;
     toplevel=FALSE;
@@ -1891,7 +2128,7 @@ BOOLEAN iiAssign(leftv l, leftv r, BOOLEAN toplevel)
       &&(lt!=INTMAT_CMD)
       &&((lt==rt)||(lt!=LIST_CMD)))
       {
-        b=jiAssign_1(l,r,toplevel);
+        b=jiAssign_1(l,r,rt,toplevel,is_qring);
         if (l->rtyp==IDHDL)
         {
           if ((lt==DEF_CMD)||(lt==LIST_CMD))
@@ -1916,7 +2153,7 @@ BOOLEAN iiAssign(leftv l, leftv r, BOOLEAN toplevel)
         &&(rt==RESOLUTION_CMD))
       )
       {
-        b=jiAssign_1(l,r,toplevel);
+        b=jiAssign_1(l,r,rt,toplevel);
         if((l->rtyp==IDHDL)&&(l->data!=NULL))
         {
           if ((lt==DEF_CMD) || (lt==LIST_CMD))
@@ -1962,21 +2199,20 @@ BOOLEAN iiAssign(leftv l, leftv r, BOOLEAN toplevel)
   }
 
   leftv hh=r;
-  BOOLEAN nok=FALSE;
   BOOLEAN map_assign=FALSE;
   switch (lt)
   {
     case INTVEC_CMD:
-      nok=jjA_L_INTVEC(l,r,new intvec(exprlist_length(r)));
+      b=jjA_L_INTVEC(l,r,new intvec(exprlist_length(r)));
       break;
     case INTMAT_CMD:
     {
-      nok=jjA_L_INTVEC(l,r,new intvec(IDINTVEC((idhdl)l->data)));
+      b=jjA_L_INTVEC(l,r,new intvec(IDINTVEC((idhdl)l->data)));
       break;
     }
     case BIGINTMAT_CMD:
     {
-      nok=jjA_L_BIGINTMAT(l, r, new bigintmat(IDBIMAT((idhdl)l->data)));
+      b=jjA_L_BIGINTMAT(l, r, new bigintmat(IDBIMAT((idhdl)l->data)));
       break;
     }
     case MAP_CMD:
@@ -1988,22 +2224,25 @@ BOOLEAN iiAssign(leftv l, leftv r, BOOLEAN toplevel)
         IDMAP((idhdl)l->data)->preimage = omStrDup (r->Fullname());
         /* advance the expressionlist to get the next element after the ring */
         hh = r->next;
-        //r=hh;
       }
       else
       {
         WerrorS("expected ring-name");
-        nok=TRUE;
+        b=TRUE;
         break;
       }
       if (hh==NULL) /* map-assign: map f=r; */
       {
         WerrorS("expected image ideal");
-        nok=TRUE;
+        b=TRUE;
         break;
       }
       if ((hh->next==NULL)&&(hh->Typ()==IDEAL_CMD))
-        return jiAssign_1(l,hh,toplevel); /* map-assign: map f=r,i; */
+      {
+        b=jiAssign_1(l,hh,IDEAL_CMD,toplevel); /* map-assign: map f=r,i; */
+        omFreeBin(hh,sleftv_bin);
+        return b;
+      }
       //no break, handle the rest like an ideal:
       map_assign=TRUE;
     }
@@ -2013,11 +2252,11 @@ BOOLEAN iiAssign(leftv l, leftv r, BOOLEAN toplevel)
     {
       sleftv t;
       matrix olm = (matrix)l->Data();
-      int rk;
+      long rk;
       char *pr=((map)olm)->preimage;
       BOOLEAN module_assign=(/*l->Typ()*/ lt==MODUL_CMD);
       matrix lm ;
-      int  num;
+      long  num;
       int j,k;
       int i=0;
       int mtyp=MATRIX_CMD; /*Type of left side object*/
@@ -2059,28 +2298,34 @@ BOOLEAN iiAssign(leftv l, leftv r, BOOLEAN toplevel)
           ht=hh->Typ();
           if ((j=iiTestConvert(ht,etyp))!=0)
           {
-            nok=iiConvert(ht,etyp,j,hh,&t);
+            b=iiConvert(ht,etyp,j,hh,&t);
             hh->next=t.next;
-            if (nok) break;
+            if (b)
+            { Werror("can not convert %s(%s) -> %s",Tok2Cmdname(ht),hh->Name(),Tok2Cmdname(etyp));
+               break;
+            }
             lm->m[i]=(poly)t.CopyD(etyp);
             pNormalize(lm->m[i]);
-            if (module_assign) rk=si_max(rk,(int)pMaxComp(lm->m[i]));
+            if (module_assign) rk=si_max(rk,pMaxComp(lm->m[i]));
             i++;
           }
           else
           if ((j=iiTestConvert(ht,mtyp))!=0)
           {
-            nok=iiConvert(ht,mtyp,j,hh,&t);
+            b=iiConvert(ht,mtyp,j,hh,&t);
             hh->next=t.next;
-            if (nok) break;
+            if (b)
+            { Werror("can not convert %s(%s) -> %s",Tok2Cmdname(ht),hh->Name(),Tok2Cmdname(mtyp));
+               break;
+            }
             rm = (matrix)t.CopyD(mtyp);
             if (module_assign)
             {
-              j = si_min(num,rm->cols());
-              rk=si_max(rk,(int)rm->rank);
+              j = si_min((int)num,rm->cols());
+              rk=si_max(rk,rm->rank);
             }
             else
-              j = si_min(num-i,rm->rows() * rm->cols());
+              j = si_min(num-i,(long)rm->rows() * (long)rm->cols());
             for(k=0;k<j;k++,i++)
             {
               lm->m[i]=rm->m[k];
@@ -2091,7 +2336,8 @@ BOOLEAN iiAssign(leftv l, leftv r, BOOLEAN toplevel)
           }
           else
           {
-            nok=TRUE;
+            b=TRUE;
+            Werror("can not convert %s(%s) -> %s",Tok2Cmdname(ht),hh->Name(),Tok2Cmdname(mtyp));
             break;
           }
           t.next=NULL;t.CleanUp();
@@ -2099,7 +2345,7 @@ BOOLEAN iiAssign(leftv l, leftv r, BOOLEAN toplevel)
           hh=hh->next;
         }
       }
-      if (nok)
+      if (b)
         idDelete((ideal *)&lm);
       else
       {
@@ -2115,29 +2361,29 @@ BOOLEAN iiAssign(leftv l, leftv r, BOOLEAN toplevel)
       break;
     }
     case STRING_CMD:
-      nok=jjA_L_STRING(l,r);
+      b=jjA_L_STRING(l,r);
       break;
     //case DEF_CMD:
     case LIST_CMD:
-      nok=jjA_L_LIST(l,r);
+      b=jjA_L_LIST(l,r);
       break;
     case NONE:
     case 0:
       Werror("cannot assign to %s",l->Fullname());
-      nok=TRUE;
+      b=TRUE;
       break;
     default:
       WerrorS("assign not impl.");
-      nok=TRUE;
+      b=TRUE;
       break;
   } /* end switch: typ */
-  if (nok && (!errorreported)) WerrorS("incompatible type in list assignment");
+  if (b && (!errorreported)) WerrorS("incompatible type in list assignment");
   r->CleanUp();
-  return nok;
+  return b;
 }
 void jjNormalizeQRingId(leftv I)
 {
-  if ((currRing->qideal!=NULL) && (!hasFlag(I,FLAG_QRING)))
+  assume ((currRing->qideal!=NULL) && (!hasFlag(I,FLAG_QRING)));
   {
     if (I->e==NULL)
     {
@@ -2170,17 +2416,21 @@ void jjNormalizeQRingId(leftv I)
     }
   }
 }
-void jjNormalizeQRingP(poly &p)
+poly jj_NormalizeQRingP(poly p, const ring r)
 {
-  if((p!=NULL) && (currRing->qideal!=NULL))
+  if((p!=NULL) && (r->qideal!=NULL))
   {
+    ring save=currRing;
+    if (r!=currRing) rChangeCurrRing(r);
     ideal F=idInit(1,1);
-    poly p2=kNF(F,currRing->qideal,p);
-    pNormalize(p2);
-    idDelete(&F);
-    pDelete(&p);
+    poly p2=kNF(F,r->qideal,p);
+    p_Normalize(p2,r);
+    id_Delete(&F,r);
+    p_Delete(&p,r);
     p=p2;
+    if (r!=save) rChangeCurrRing(save);
   }
+  return p;
 }
 BOOLEAN jjIMPORTFROM(leftv, leftv u, leftv v)
 {
