@@ -10,82 +10,63 @@
 
 
 
-#include <kernel/mod2.h>
+#include "kernel/mod2.h"
 
-#include <omalloc/omalloc.h>
+#include "misc/options.h"
+#include "misc/intvec.h"
 
-#include <misc/options.h>
-#include <misc/intvec.h>
+#include "coeffs/numbers.h"
+#include "coeffs/bigintmat.h"
 
-#include <coeffs/numbers.h>
-#include <coeffs/bigintmat.h>
+#include "polys/matpol.h"
+#include "polys/monomials/ring.h"
 
-#include <polys/matpol.h>
-#include <polys/monomials/ring.h>
+#include "kernel/polys.h"
+#include "kernel/ideals.h"
+#include "kernel/GBEngine/syz.h"
 
-#include <kernel/polys.h>
-#include <kernel/ideals.h>
-#include <kernel/GBEngine/syz.h>
-
-#include <Singular/tok.h>
-#include <Singular/ipshell.h>
-#include <Singular/fevoices.h>
-#include <Singular/lists.h>
-#include <Singular/attrib.h>
-#include <Singular/links/silink.h>
-#include <Singular/ipid.h>
-#include <Singular/blackbox.h>
-#include <Singular/number2.h>
+#include "Singular/tok.h"
+#include "Singular/ipshell.h"
+#include "Singular/fevoices.h"
+#include "Singular/lists.h"
+#include "Singular/attrib.h"
+#include "Singular/links/silink.h"
+#include "Singular/ipid.h"
+#include "Singular/blackbox.h"
+#include "Singular/number2.h"
 
 #ifdef SINGULAR_4_2
-#include <Singular/number2.h>
+#include "Singular/number2.h"
 #endif
 #ifdef HAVE_DYNAMIC_LOADING
-#include <polys/mod_raw.h>
+#include "polys/mod_raw.h"
 #endif /* HAVE_DYNAMIC_LOADING */
 
-#include <string.h>
-
-omBin sip_command_bin = omGetSpecBin(sizeof(sip_command));
-omBin sip_package_bin = omGetSpecBin(sizeof(sip_package));
+VAR omBin sip_command_bin = omGetSpecBin(sizeof(sip_command));
+VAR omBin sip_package_bin = omGetSpecBin(sizeof(sip_package));
 //omBin ip_package_bin = omGetSpecBin(sizeof(ip_package));
-omBin idrec_bin = omGetSpecBin(sizeof(idrec));
+VAR omBin idrec_bin = omGetSpecBin(sizeof(idrec));
 
-coeffs coeffs_BIGINT;
+VAR coeffs coeffs_BIGINT;
 
-FILE   *feFilePending; /*temp. storage for grammar.y */
-
-proclevel *procstack=NULL;
+VAR proclevel *procstack=NULL;
 //idhdl idroot = NULL;
 
-idhdl currPackHdl = NULL;
-idhdl basePackHdl = NULL;
-package currPack =NULL;
-package basePack =NULL;
-idhdl currRingHdl = NULL;
-const char* iiNoName="_";
+VAR idhdl currPackHdl = NULL;
+VAR idhdl basePackHdl = NULL;
+VAR package currPack = NULL;
+VAR package basePack = NULL;
+VAR idhdl currRingHdl = NULL;
 
 void paCleanUp(package pack);
 
 /*0 implementation*/
 
-int iiS2I(const char *s)
+static inline long iiS2I(const char *s)
 {
-  int i;
-  i=s[0];
-  if (s[1]!='\0')
-  {
-    i=(i<<8)+s[1];
-    if (s[2]!='\0')
-    {
-      i=(i<<8)+s[2];
-      if (s[3]!='\0')
-      {
-        i=(i<<8)+s[3];
-      }
-    }
-  }
-  return i;
+  long l;
+  strncpy((char*)&l,s,SIZEOF_LONG);
+  return l;
 }
 
 idhdl idrec::get(const char * s, int level)
@@ -96,8 +77,8 @@ idhdl idrec::get(const char * s, int level)
   idhdl found=NULL;
   int l;
   const char *id_;
-  int i=iiS2I(s);
-  int less4=(i < (1<<24));
+  unsigned long i=iiS2I(s);
+  int less4=(i < (1L<<((SIZEOF_LONG-1)*8)));
   while (h!=NULL)
   {
     omCheckAddr((ADDRESS)IDID(h));
@@ -107,9 +88,9 @@ idhdl idrec::get(const char * s, int level)
       if (i==h->id_i)
       {
         id_=IDID(h);
-        if (less4 || (0 == strcmp(s+4,id_+4)))
+        if (less4 || (0 == strcmp(s+SIZEOF_LONG,id_+SIZEOF_LONG)))
         {
-          if (l==level) return h;
+          if(l==level) return h;
           found=h;
         }
       }
@@ -119,6 +100,32 @@ idhdl idrec::get(const char * s, int level)
   return found;
 }
 
+idhdl idrec::get_level(const char * s, int level)
+{
+  assume(s!=NULL);
+  assume((level>=0) && (level<=1000)); //not really, but if it isnt in that bounds..
+  idhdl h = this;
+  int l;
+  const char *id_;
+  unsigned long i=iiS2I(s);
+  int less4=(i < (1L<<((SIZEOF_LONG-1)*8)));
+  while (h!=NULL)
+  {
+    omCheckAddr((ADDRESS)IDID(h));
+    l=IDLEV(h);
+    if ((l==level)&&(i==h->id_i))
+    {
+      id_=IDID(h);
+      if (less4 || (0 == strcmp(s+SIZEOF_LONG,id_+SIZEOF_LONG)))
+      {
+        return h;
+      }
+    }
+    h = IDNEXT(h);
+  }
+  return NULL;
+}
+
 //idrec::~idrec()
 //{
 //  if (id!=NULL)
@@ -143,16 +150,28 @@ void *idrecDataInit(int t)
 #endif
     case BIGINTMAT_CMD:
       return (void *)new bigintmat();
+    case BUCKET_CMD:
+      if (currRing!=NULL)
+      return (void*)sBucketCreate(currRing);
+      else
+      {
+        WerrorS("need basering for polyBucket");
+        return NULL;
+      }
     case INTVEC_CMD:
     case INTMAT_CMD:
       return (void *)new intvec();
     case NUMBER_CMD:
-      return (void *) nInit(0);
+    {
+      if (currRing!=NULL) return (void *) nInit(0);
+      else                return NULL;
+    }
     case BIGINT_CMD:
       return (void *) n_Init(0, coeffs_BIGINT);
     case IDEAL_CMD:
     case MODUL_CMD:
     case MATRIX_CMD:
+    case SMATRIX_CMD:
       return (void*) idInit(1,1);
     case MAP_CMD:
     {
@@ -171,8 +190,6 @@ void *idrecDataInit(int t)
     //the types with the standard init: set the struct to zero
     case LINK_CMD:
       return (void*) omAlloc0Bin(sip_link_bin);
-    case RING_CMD:
-      return NULL;
     case PACKAGE_CMD:
     {
       package pa=(package)omAlloc0Bin(sip_package_bin);
@@ -189,14 +206,15 @@ void *idrecDataInit(int t)
     }
     case RESOLUTION_CMD:
       return  (void *)omAlloc0(sizeof(ssyStrategy));
-    //other types: without init (int,script,poly,def,package)
-    case CRING_CMD:
+    //other types: without alloc. (int,script,poly,def,package,..)
     case INT_CMD:
     case DEF_CMD:
     case POLY_CMD:
     case VECTOR_CMD:
+    case RING_CMD:
+    case CRING_CMD:
     case QRING_CMD:
-       return (void*)0L;
+      return NULL;
     default:
       {
         if (t>MAX_TOK)
@@ -225,6 +243,7 @@ idhdl idrec::set(const char * s, int level, int t, BOOLEAN init)
   IDNEXT(h) = this;
   BOOLEAN at_start=(this==IDROOT);
   h->id_i=iiS2I(s);
+  if (t==BUCKET_CMD) WarnS("defining polyBucket");
   if (init)
   {
     if ((t==IDEAL_CMD)||(t==MODUL_CMD))
@@ -249,7 +268,7 @@ idhdl idrec::set(const char * s, int level, int t, BOOLEAN init)
 char * idrec::String(BOOLEAN typed)
 {
   sleftv tmp;
-  memset(&tmp,0,sizeof(sleftv));
+  tmp.Init();
   tmp.rtyp=IDTYP(this);
   tmp.data=IDDATA(this);
   tmp.name=IDID(this);
@@ -271,68 +290,77 @@ idhdl enterid(const char * s, int lev, int t, idhdl* root, BOOLEAN init, BOOLEAN
     }
   }
   // is it already defined in root ?
-  if ((h=(*root)->get(s,lev))!=NULL)
+  if ((h=(*root)->get_level(s,lev))!=NULL)
   {
-    if (IDLEV(h)==lev)
+    if ((IDTYP(h) == t)||(t==DEF_CMD))
+    {
+      if (IDTYP(h)==PACKAGE_CMD)
+      {
+        if (strcmp(s,"Top")==0)
+        {
+          goto errlabel;
+        }
+        else return h;
+      }
+      else
+      {
+        if (BVERBOSE(V_REDEFINE))
+        {
+          const char *f=VoiceName();
+          if (strcmp(f,"STDIN")==0)
+            Warn("redefining %s (%s)",s,my_yylinebuf);
+          else
+            Warn("redefining %s (%s) %s:%d",s,my_yylinebuf,f, yylineno);
+        }
+        if (s==IDID(h)) IDID(h)=NULL;
+        killhdl2(h,root,currRing);
+      }
+    }
+    else
+      goto errlabel;
+  }
+  // is it already defined in currRing->idroot ?
+  else if (search && (currRing!=NULL)&&((*root) != currRing->idroot))
+  {
+    if ((h=currRing->idroot->get_level(s,lev))!=NULL)
     {
       if ((IDTYP(h) == t)||(t==DEF_CMD))
       {
-        if (IDTYP(h)==PACKAGE_CMD)
+        if (BVERBOSE(V_REDEFINE))
         {
-          if (strcmp(s,"Top")==0)
-          {
-            goto errlabel;
-          }
-          else return *root;
-        }
-        else
-        {
-          if (BVERBOSE(V_REDEFINE))
+          const char *f=VoiceName();
+          if (strcmp(f,"STDIN")==0)
             Warn("redefining %s (%s)",s,my_yylinebuf);
-          if (s==IDID(h)) IDID(h)=NULL;
-          killhdl2(h,root,currRing);
+          else
+            Warn("redefining %s (%s) %s:%d",s,my_yylinebuf,f, yylineno);
         }
+        if (s==IDID(h)) IDID(h)=NULL;
+        killhdl2(h,&currRing->idroot,currRing);
       }
       else
         goto errlabel;
     }
   }
-  // is it already defined in currRing->idroot ?
-  else if (search && (currRing!=NULL)&&((*root) != currRing->idroot))
-  {
-    if ((h=currRing->idroot->get(s,lev))!=NULL)
-    {
-      if (IDLEV(h)==lev)
-      {
-        if ((IDTYP(h) == t)||(t==DEF_CMD))
-        {
-          if (BVERBOSE(V_REDEFINE))
-            Warn("redefining %s (%s)",s,my_yylinebuf);
-          if (s==IDID(h)) IDID(h)=NULL;
-          killhdl2(h,&currRing->idroot,currRing);
-        }
-        else
-          goto errlabel;
-      }
-    }
-  }
   // is it already defined in idroot ?
   else if (search && (*root != IDROOT))
   {
-    if ((h=IDROOT->get(s,lev))!=NULL)
+    if ((h=IDROOT->get_level(s,lev))!=NULL)
     {
-      if (IDLEV(h)==lev)
+      if ((IDTYP(h) == t)||(t==DEF_CMD))
       {
-        if ((IDTYP(h) == t)||(t==DEF_CMD))
+        if (BVERBOSE(V_REDEFINE))
         {
-          if (BVERBOSE(V_REDEFINE))
+          const char *f=VoiceName();
+          if (strcmp(f,"STDIN")==0)
             Warn("redefining %s (%s)",s,my_yylinebuf);
-          if (s==IDID(h)) IDID(h)=NULL;
-          killhdl2(h,&IDROOT,NULL);
+          else
+            Warn("redefining %s (%s) %s:%d",s,my_yylinebuf,f, yylineno);
         }
-        else
-          goto errlabel;
+        if (s==IDID(h)) IDID(h)=NULL;
+        killhdl2(h,&IDROOT,NULL);
       }
+      else
+        goto errlabel;
     }
   }
   *root = (*root)->set(s, lev, t, init);
@@ -421,14 +449,18 @@ void killhdl2(idhdl h, idhdl * ih, ring r)
   }
   if (h->attribute!=NULL)
   {
-    //h->attribute->killAll(r); MEMORY LEAK!
+    if ((IDTYP(h)==RING_CMD)&&(IDRING(h)!=r))
+       h->attribute->killAll(IDRING(h));
+    else
+       h->attribute->killAll(r);
     h->attribute=NULL;
   }
   if (IDTYP(h) == PACKAGE_CMD)
   {
-    if (strcmp(IDID(h),"Top")==0)
+    if (((IDPACKAGE(h)->language==LANG_C)&&(IDPACKAGE(h)->idroot!=NULL))
+    || (strcmp(IDID(h),"Top")==0))
     {
-      WarnS("can not kill `Top`");
+      Warn("cannot kill `%s`",IDID(h));
       return;
     }
     // any objects defined for this package ?
@@ -493,6 +525,7 @@ void killhdl2(idhdl h, idhdl * ih, ring r)
   omFreeBin((ADDRESS)h, idrec_bin);
 }
 
+#if 0
 idhdl ggetid(const char *n, BOOLEAN /*local*/, idhdl *packhdl)
 {
   idhdl h = IDROOT->get(n,myynest);
@@ -505,22 +538,73 @@ idhdl ggetid(const char *n, BOOLEAN /*local*/, idhdl *packhdl)
   if (h2==NULL) return h;
   return h2;
 }
+#endif
 
+#if 0
+// debug version
+idhdl ggetid(const char *n)
+{
+  if (currRing!=NULL)
+  {
+    idhdl h2 = currRing->idroot->get(n,myynest);
+    idhdl h = IDROOT->get(n,myynest);
+    if ((h!=NULL)&&(h2!=NULL)&&(IDLEV(h)==IDLEV(h2)))
+    {
+      Warn("SHADOW %s(%s) vs %s(%s) in %s\n",IDID(h),Tok2Cmdname(IDTYP(h)),IDID(h2),Tok2Cmdname(IDTYP(h2)),my_yylinebuf);
+    }
+    if ((h2!=NULL)&&(IDLEV(h2)==myynest)) return h2;
+    if (h!=NULL) return h;
+    if (h2!=NULL) return h2;
+  }
+  else
+  {
+    idhdl h = IDROOT->get(n,myynest);
+    if (h!=NULL) return h;
+  }
+  if (basePack!=currPack)
+    return basePack->idroot->get(n,myynest);
+  return NULL;
+}
+#endif
+#if 1
+// try currRing before non-ring stuff
+idhdl ggetid(const char *n)
+{
+  if (currRing!=NULL)
+  {
+    idhdl h2 = currRing->idroot->get(n,myynest);
+    if ((h2!=NULL)&&(IDLEV(h2)==myynest)) return h2;
+    idhdl h = IDROOT->get(n,myynest);
+    if (h!=NULL) return h;
+    if (h2!=NULL) return h2;
+  }
+  else
+  {
+    idhdl h = IDROOT->get(n,myynest);
+    if (h!=NULL) return h;
+  }
+  if (basePack!=currPack)
+    return basePack->idroot->get(n,myynest);
+  return NULL;
+}
+#endif
+#if 0
+// try non-ring stuff before ring stuff
 idhdl ggetid(const char *n)
 {
   idhdl h = IDROOT->get(n,myynest);
   if ((h!=NULL)&&(IDLEV(h)==myynest)) return h;
-  idhdl h2=NULL;
   if (currRing!=NULL)
   {
-    h2 = currRing->idroot->get(n,myynest);
+    idhdl h2 = currRing->idroot->get(n,myynest);
+    if (h2!=NULL) return h2;
   }
-  if (h2!=NULL) return h2;
   if (h!=NULL) return h;
   if (basePack!=currPack)
     return basePack->idroot->get(n,myynest);
   return NULL;
 }
+#endif
 
 void ipListFlag(idhdl h)
 {
