@@ -5,29 +5,19 @@
 * ABSTRACT: interpreter: LIB and help
 */
 
+#include "kernel/mod2.h"
 
+#include "Singular/tok.h"
+#include "misc/options.h"
+#include "Singular/ipid.h"
+#include "polys/monomials/ring.h"
+#include "Singular/subexpr.h"
+#include "Singular/ipid.h"
+#include "Singular/ipshell.h"
+#include "Singular/fevoices.h"
+#include "Singular/lists.h"
 
-
-
-#include <kernel/mod2.h>
-
-#include <Singular/tok.h>
-#include <misc/options.h>
-#include <Singular/ipid.h>
-#include <omalloc/omalloc.h>
-#include <polys/monomials/ring.h>
-#include <Singular/subexpr.h>
-#include <Singular/ipid.h>
-#include <Singular/ipshell.h>
-#include <Singular/fevoices.h>
-#include <Singular/lists.h>
-
-//#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
 #include <ctype.h>
-#include <sys/stat.h>
-
 
 #if SIZEOF_LONG == 8
 #define SI_MAX_NEST 500
@@ -62,16 +52,16 @@ procinfo *iiInitSingularProcinfo(procinfov pi, const char *libname,
 extern int iiArithAddCmd(const char *szName, short nAlias, short nTokval,
                          short nToktype, short nPos);
 
-#include <Singular/mod_lib.h>
+#include "Singular/mod_lib.h"
 
 #ifdef HAVE_LIBPARSER
 void yylprestart (FILE *input_file );
 int current_pos(int i=0);
-extern int yylp_errno;
-extern int yylplineno;
-extern char *yylp_errlist[];
+EXTERN_VAR int yylp_errno;
+EXTERN_VAR int yylplineno;
+extern const char *yylp_errlist[];
 void print_init();
-libstackv library_stack;
+VAR libstackv library_stack;
 #endif
 
 //int IsCmd(char *n, int tok);
@@ -80,7 +70,7 @@ char mytolower(char c);
 /*2
 * return TRUE if the libray libname is already loaded
 */
-BOOLEAN iiGetLibStatus(char *lib)
+BOOLEAN iiGetLibStatus(const char *lib)
 {
   idhdl hl;
 
@@ -91,16 +81,9 @@ BOOLEAN iiGetLibStatus(char *lib)
   {
     return FALSE;
   }
-  return (strcmp(lib,IDPACKAGE(hl)->libname)==0);
-}
-
-/*2
-* find the library of an proc:
-*  => return (pi->libname)
-*/
-char * iiGetLibName(procinfov pi)
-{
-  return pi->libname;
+  if ((IDPACKAGE(hl)->language!=LANG_C)&&(IDPACKAGE(hl)->libname!=NULL))
+    return (strcmp(lib,IDPACKAGE(hl)->libname)==0);
+  return FALSE;
 }
 
 /*2
@@ -308,18 +291,31 @@ char* iiGetLibProcBuffer(procinfo *pi, int part )
   return NULL;
 }
 
-BOOLEAN iiAllStart(procinfov pi, char *p,feBufferTypes t, int l)
+BOOLEAN iiAllStart(procinfov pi, const char *p, feBufferTypes t, int l)
 {
+  int save_trace=traceit;
+  int restore_traceit=0;
+  if (traceit_stop
+  && (traceit & TRACE_SHOW_LINE))
+  {
+    traceit &=(~TRACE_SHOW_LINE);
+    traceit_stop=0;
+    restore_traceit=1;
+  }
   // see below:
   BITSET save1=si_opt_1;
   BITSET save2=si_opt_2;
   newBuffer( omStrDup(p /*pi->data.s.body*/), t /*BT_proc*/,
                pi, l );
   BOOLEAN err=yyparse();
+
   if (sLastPrinted.rtyp!=0)
   {
     sLastPrinted.CleanUp();
   }
+
+  if (restore_traceit) traceit=save_trace;
+
   // the access to optionStruct and verboseStruct do not work
   // on x86_64-Linux for pic-code
   if ((TEST_V_ALLWARN) &&
@@ -368,7 +364,7 @@ BOOLEAN iiAllStart(procinfov pi, char *p,feBufferTypes t, int l)
 * TODO:interrupt
 * return FALSE on success, TRUE if an error occurs
 */
-BOOLEAN iiPStart(idhdl pn, sleftv  * v)
+BOOLEAN iiPStart(idhdl pn, leftv v)
 {
   procinfov pi=NULL;
   int old_echo=si_echo;
@@ -395,17 +391,18 @@ BOOLEAN iiPStart(idhdl pn, sleftv  * v)
   }
   else return TRUE;
   /* generate argument list ======================================*/
+  //iiCurrArgs should be NULL here, as the assignment for the parameters
+  // of the prevouis call are already done befor calling another routine
   if (v!=NULL)
   {
     iiCurrArgs=(leftv)omAllocBin(sleftv_bin);
-    memcpy(iiCurrArgs,v,sizeof(sleftv));
-    memset(v,0,sizeof(sleftv));
+    memcpy(iiCurrArgs,v,sizeof(sleftv)); // keeps track of v->next etc.
+    v->Init();
   }
   else
   {
     iiCurrArgs=NULL;
   }
-  iiCurrProc=pn;
   /* start interpreter ======================================*/
   myynest++;
   if (myynest > SI_MAX_NEST)
@@ -415,7 +412,9 @@ BOOLEAN iiPStart(idhdl pn, sleftv  * v)
   }
   else
   {
+    iiCurrProc=pn;
     err=iiAllStart(pi,pi->data.s.body,BT_proc,pi->data.s.body_lineno-(v!=NULL));
+    iiCurrProc=NULL;
 
     if (iiLocalRing[myynest-1] != currRing)
     {
@@ -433,7 +432,7 @@ BOOLEAN iiPStart(idhdl pn, sleftv  * v)
           nh=rFindHdl(currRing,NULL);
         if (nh!=NULL)          n=nh->id;
         else                   n="none";
-        Werror("ring change during procedure call: %s -> %s (level %d)",o,n,myynest);
+        Werror("ring change during procedure call %s: %s -> %s (level %d)",pi->procname,o,n,myynest);
         iiRETURNEXPR.CleanUp();
         err=TRUE;
       }
@@ -467,9 +466,9 @@ BOOLEAN iiPStart(idhdl pn, sleftv  * v)
   return err;
 }
 
-ring    *iiLocalRing;
-sleftv  iiRETURNEXPR;
-int     iiRETURNEXPR_len=0;
+VAR ring    *iiLocalRing;
+INST_VAR sleftv  iiRETURNEXPR;
+VAR int     iiRETURNEXPR_len=0;
 
 #ifdef RDEBUG
 static void iiShowLevRings()
@@ -498,7 +497,7 @@ static void iiCheckNest()
     iiRETURNEXPR_len+=16;
   }
 }
-BOOLEAN iiMake_proc(idhdl pn, package pack, sleftv* sl)
+BOOLEAN iiMake_proc(idhdl pn, package pack, leftv args)
 {
   int err;
   procinfov pi = IDPROC(pn);
@@ -545,11 +544,11 @@ BOOLEAN iiMake_proc(idhdl pn, package pack, sleftv* sl)
                    currPackHdl=packFindHdl(currPack);
                    //Print("set pack=%s\n",IDID(currPackHdl));
                  }
-                 err=iiPStart(pn,sl);
+                 err=iiPStart(pn,args);
                  break;
     case LANG_C:
                  leftv res = (leftv)omAlloc0Bin(sleftv_bin);
-                 err = (pi->data.o.function)(res, sl);
+                 err = (pi->data.o.function)(res, args);
                  memcpy(&iiRETURNEXPR,res,sizeof(iiRETURNEXPR));
                  omFreeBin((ADDRESS)res,  sleftv_bin);
                  break;
@@ -583,7 +582,167 @@ BOOLEAN iiMake_proc(idhdl pn, package pack, sleftv* sl)
     return TRUE;
   return FALSE;
 }
+static void iiCallLibProcBegin()
+{
+  idhdl tmp_ring=NULL;
+  if (currRing!=NULL)
+  {
+    if ((currRingHdl!=NULL) && (IDRING(currRingHdl)!=currRing))
+    {
+      // clean up things depending on currRingHdl:
+      sLastPrinted.CleanUp(IDRING(currRingHdl));
+      sLastPrinted.Init();
+    }
+    // need to define a ring-hdl for currRingHdl
+    tmp_ring=enterid(" tmpRing",myynest,RING_CMD,&IDROOT,FALSE);
+    IDRING(tmp_ring)=rIncRefCnt(currRing);
+    rSetHdl(tmp_ring);
+  }
+}
+static void iiCallLibProcEnd(idhdl save_ringhdl, ring save_ring)
+{
+  if ((currRing!=NULL)
+  &&(currRing!=save_ring))
+  {
+    rDecRefCnt(currRing);
+    idhdl hh=IDROOT;
+    idhdl prev=NULL;
+    while((hh!=currRingHdl) && (hh!=NULL)) { prev=hh; hh=hh->next; }
+    if (hh!=NULL)
+    {
+      if (prev==NULL) IDROOT=hh->next;
+      else prev->next=hh->next;
+      omFree((ADDRESS)IDID(hh));
+      omFreeBin((ADDRESS)hh, idrec_bin);
+    }
+  }
+  currRingHdl=save_ringhdl;
+  currRing=save_ring;
+}
 
+void* iiCallLibProc1(const char*n, void *arg, int arg_type, BOOLEAN &err)
+{
+  idhdl h=ggetid(n);
+  if ((h==NULL)
+  || (IDTYP(h)!=PROC_CMD))
+  {
+    err=2;
+    return NULL;
+  }
+  // ring handling
+  idhdl save_ringhdl=currRingHdl;
+  ring save_ring=currRing;
+  iiCallLibProcBegin();
+  // argument:
+  sleftv tmp;
+  tmp.Init();
+  tmp.data=arg;
+  tmp.rtyp=arg_type;
+  // call proc
+  err=iiMake_proc(h,currPack,&tmp);
+  // clean up ring
+  iiCallLibProcEnd(save_ringhdl,save_ring);
+  // return
+  if (err==FALSE)
+  {
+    void*r=iiRETURNEXPR.data;
+    iiRETURNEXPR.data=NULL;
+    iiRETURNEXPR.CleanUp();
+    return r;
+  }
+  return NULL;
+}
+
+// return NULL on failure
+ideal ii_CallProcId2Id(const char *lib,const char *proc, ideal arg, const ring R)
+{
+  char *plib = iiConvName(lib);
+  idhdl h=ggetid(plib);
+  omFree(plib);
+  if (h==NULL)
+  {
+    BOOLEAN bo=iiLibCmd(lib,TRUE,TRUE,FALSE);
+    if (bo) return NULL;
+  }
+  ring oldR=currRing;
+  rChangeCurrRing(R);
+  BOOLEAN err;
+  ideal I=(ideal)iiCallLibProc1(proc,idCopy(arg),IDEAL_CMD,err);
+  rChangeCurrRing(oldR);
+  if (err) return NULL;
+  return I;
+}
+
+int ii_CallProcId2Int(const char *lib,const char *proc, ideal arg, const ring R)
+{
+  char *plib = iiConvName(lib);
+  idhdl h=ggetid(plib);
+  omFree(plib);
+  if (h==NULL)
+  {
+    BOOLEAN bo=iiLibCmd(lib,TRUE,TRUE,FALSE);
+    if (bo) return 0;
+  }
+  BOOLEAN err;
+  ring oldR=currRing;
+  rChangeCurrRing(R);
+  int I=(int)(long)iiCallLibProc1(proc,idCopy(arg),IDEAL_CMD,err);
+  rChangeCurrRing(oldR);
+  if (err) return 0;
+  return I;
+}
+
+/// args: NULL terminated array of arguments
+/// arg_types: 0 terminated array of corresponding types
+leftv ii_CallLibProcM(const char*n, void **args, int* arg_types, const ring R, BOOLEAN &err)
+{
+  idhdl h=ggetid(n);
+  if ((h==NULL)
+  || (IDTYP(h)!=PROC_CMD))
+  {
+    err=2;
+    return NULL;
+  }
+  // ring handling
+  idhdl save_ringhdl=currRingHdl;
+  ring save_ring=currRing;
+  rChangeCurrRing(R);
+  iiCallLibProcBegin();
+  // argument:
+  if (arg_types[0]!=0)
+  {
+    sleftv tmp;
+    leftv tt=&tmp;
+    int i=1;
+    tmp.Init();
+    tmp.data=args[0];
+    tmp.rtyp=arg_types[0];
+    while(arg_types[i]!=0)
+    {
+      tt->next=(leftv)omAlloc0Bin(sleftv_bin);
+      tt=tt->next;
+      tt->rtyp=arg_types[i];
+      tt->data=args[i];
+      i++;
+    }
+    // call proc
+    err=iiMake_proc(h,currPack,&tmp);
+  }
+  else
+  // call proc
+    err=iiMake_proc(h,currPack,NULL);
+  // clean up ring
+  iiCallLibProcEnd(save_ringhdl,save_ring);
+  // return
+  if (err==FALSE)
+  {
+    leftv h=(leftv)omAllocBin(sleftv_bin);
+    memcpy(h,&iiRETURNEXPR,sizeof(sleftv));
+    iiRETURNEXPR.Init();
+    return h;
+  }
+  return NULL;
+}
 /*2
 * start an example (as a proc),
 * destroys the string 'example'
@@ -638,10 +797,14 @@ extern "C"
 #  undef  SI_GET_BUILTIN_MOD_INIT0
 };
 
+extern "C" int flint_mod_init(SModulFunctions* psModulFunctions);
 
 SModulFunc_t
 iiGetBuiltinModInit(const char* libname)
 {
+#ifdef HAVE_FLINT
+  if (strcmp(libname,"flint.so")==0) return SI_MOD_INIT0(flint);
+#endif
 #  define SI_GET_BUILTIN_MOD_INIT(name) if (strcmp(libname, #name ".so") == 0){ return SI_MOD_INIT0(name); }
           SI_FOREACH_BUILTIN(SI_GET_BUILTIN_MOD_INIT)
 #  undef  SI_GET_BUILTIN_MOD_INIT
@@ -670,20 +833,19 @@ BOOLEAN iiTryLoadLib(leftv v, const char *id)
     *libname = mytolower(*libname);
     if((LT = type_of_LIB(libname, libnamebuf)) > LT_NOTFOUND)
     {
-      char *s=omStrDup(libname);
       #ifdef HAVE_DYNAMIC_LOADING
       char libnamebuf[1024];
       #endif
 
       if (LT==LT_SINGULAR)
-        LoadResult = iiLibCmd(s, FALSE, FALSE,TRUE);
+        LoadResult = iiLibCmd(libname, FALSE, FALSE,TRUE);
       #ifdef HAVE_DYNAMIC_LOADING
       else if ((LT==LT_ELF) || (LT==LT_HPUX))
-        LoadResult = load_modules(s,libnamebuf,FALSE);
+        LoadResult = load_modules(libname,libnamebuf,FALSE);
       #endif
       else if (LT==LT_BUILTIN)
       {
-        LoadResult=load_builtin(s,FALSE, iiGetBuiltinModInit(s));
+        LoadResult=load_builtin(libname,FALSE, iiGetBuiltinModInit(libname));
       }
       if(!LoadResult )
       {
@@ -715,14 +877,11 @@ BOOLEAN iiLocateLib(const char* lib, char* where)
     return FALSE;;
 }
 
-BOOLEAN iiLibCmd( char *newlib, BOOLEAN autoexport, BOOLEAN tellerror, BOOLEAN force )
+BOOLEAN iiLibCmd( const char *newlib, BOOLEAN autoexport, BOOLEAN tellerror, BOOLEAN force )
 {
+  if (strcmp(newlib,"Singular")==0) return FALSE;
   char libnamebuf[1024];
-  // procinfov pi;
-  // idhdl h;
   idhdl pl;
-  // idhdl hl;
-  // long pos = 0L;
   char *plib = iiConvName(newlib);
   FILE * fp = feFopen( newlib, "r", libnamebuf, tellerror );
   // int lines = 1;
@@ -744,19 +903,22 @@ BOOLEAN iiLibCmd( char *newlib, BOOLEAN autoexport, BOOLEAN tellerror, BOOLEAN f
   {
     if(IDTYP(pl)!=PACKAGE_CMD)
     {
+      omFree(plib);
       WarnS("not of type package.");
       fclose(fp);
       return TRUE;
     }
-    if (!force) return FALSE;
+    if (!force)
+    {
+      omFree(plib);
+      return FALSE;
+    }
   }
   LoadResult = iiLoadLIB(fp, libnamebuf, newlib, pl, autoexport, tellerror);
-  omFree((ADDRESS)newlib);
 
   if(!LoadResult) IDPACKAGE(pl)->loaded = TRUE;
   omFree((ADDRESS)plib);
-
- return LoadResult;
+  return LoadResult;
 }
 /*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
 static void iiCleanProcs(idhdl &root)
@@ -807,7 +969,7 @@ static void iiRunInit(package p)
 BOOLEAN iiLoadLIB(FILE *fp, const char *libnamebuf, const char*newlib,
              idhdl pl, BOOLEAN autoexport, BOOLEAN tellerror)
 {
-  extern FILE *yylpin;
+  EXTERN_VAR FILE *yylpin;
   libstackv ls_start = library_stack;
   lib_style_types lib_style;
 
@@ -815,7 +977,7 @@ BOOLEAN iiLoadLIB(FILE *fp, const char *libnamebuf, const char*newlib,
   #if YYLPDEBUG > 1
   print_init();
   #endif
-  extern int lpverbose;
+  EXTERN_VAR int lpverbose;
   if (BVERBOSE(V_DEBUG_LIB)) lpverbose=1;
   else lpverbose=0;
   // yylplex sets also text_buffer
@@ -844,8 +1006,8 @@ BOOLEAN iiLoadLIB(FILE *fp, const char *libnamebuf, const char*newlib,
   if( (lib_style == OLD_LIBSTYLE) && (BVERBOSE(V_LOAD_LIB)))
   {
     Warn( "library %s has old format. This format is still accepted,", newlib);
-    Warn( "but for functionality you may wish to change to the new");
-    Warn( "format. Please refer to the manual for further information.");
+    WarnS( "but for functionality you may wish to change to the new");
+    WarnS( "format. Please refer to the manual for further information.");
   }
   reinit_yylp();
   fclose( yylpin );
@@ -914,9 +1076,11 @@ int iiAddCproc(const char *libname, const char *procname, BOOLEAN pstatic,
   && (IDTYP(h)==PROC_CMD))
   {
     pi = IDPROC(h);
+    #if 0
     if ((pi->language == LANG_SINGULAR)
     &&(BVERBOSE(V_REDEFINE)))
       Warn("extend `%s`",procname);
+    #endif
   }
   else
   {
@@ -957,6 +1121,7 @@ int iiAddCproc(const char *libname, const char *procname, BOOLEAN pstatic,
     }
     else
       Warn("internal error: unknown procedure type %d",pi->language);
+    if (currPack->language==LANG_SINGULAR) currPack->language=LANG_MIX;
     return(1);
   }
   else
@@ -979,12 +1144,38 @@ int iiAddCprocTop(const char *libname, const char *procname, BOOLEAN pstatic,
 
 /*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
 #ifdef HAVE_DYNAMIC_LOADING
-BOOLEAN load_modules(const char *newlib, char *fullname, BOOLEAN autoexport)
+#include <map>
+#include <string>
+#include <pthread.h>
+
+THREAD_VAR std::map<std::string, void *> *dyn_modules;
+
+bool registered_dyn_module(char *fullname) {
+  if (dyn_modules == NULL)
+    return false;
+  std::string fname = fullname;
+  return dyn_modules->count(fname) != 0;
+}
+
+void register_dyn_module(char *fullname, void * handle) {
+  std::string fname = fullname;
+  if (dyn_modules == NULL)
+    dyn_modules = new std::map<std::string, void *>();
+  dyn_modules->insert(std::pair<std::string, void *>(fname, handle));
+}
+
+void close_all_dyn_modules() {
+  for (std::map<std::string, void *>::iterator it = dyn_modules->begin();
+       it != dyn_modules->end();
+       it++)
+  {
+    dynl_close(it->second);
+  }
+  delete dyn_modules;
+  dyn_modules = NULL;
+}
+BOOLEAN load_modules_aux(const char *newlib, char *fullname, BOOLEAN autoexport)
 {
-#ifdef HAVE_STATIC
-  WerrorS("mod_init: static version can not load modules");
-  return TRUE;
-#else
 /*
   typedef int (*fktn_t)(int(*iiAddCproc)(const char *libname, const char *procname,
                                BOOLEAN pstatic,
@@ -997,7 +1188,7 @@ BOOLEAN load_modules(const char *newlib, char *fullname, BOOLEAN autoexport)
   int token;
   char FullName[256];
 
-  memset(FullName,0,256);
+  memset(FullName,0,sizeof(FullName));
 
   if( *fullname != '/' &&  *fullname != '.' )
     sprintf(FullName, "./%s", newlib);
@@ -1020,10 +1211,17 @@ BOOLEAN load_modules(const char *newlib, char *fullname, BOOLEAN autoexport)
       omFree(plib);
       return FALSE;
     }
+    else if(IDPACKAGE(pl)->language==LANG_MIX)
+    {
+      if (BVERBOSE(V_LOAD_LIB)) Warn( "%s contain binary parts, cannot load", newlib);
+      omFree(plib);
+      return FALSE;
+    }
   }
   else
   {
     pl = enterid( plib,0, PACKAGE_CMD, &IDROOT, TRUE );
+    omFree(plib); /* enterid copied plib*/
     IDPACKAGE(pl)->libname=omStrDup(newlib);
   }
   IDPACKAGE(pl)->language = LANG_C;
@@ -1061,19 +1259,30 @@ BOOLEAN load_modules(const char *newlib, char *fullname, BOOLEAN autoexport)
         Warn("loaded %s for a different version of Singular(expected MAX_TOK: %d, got %d)",fullname,MAX_TOK,ver);
       }
       currPack->loaded=1;
-      currPack=s;
+      currPack=s; /* reset currPack to previous */
+      register_dyn_module(fullname, IDPACKAGE(pl)->handle);
       RET=FALSE;
     }
     else
     {
       Werror("mod_init not found:: %s\nThis is probably not a dynamic module for Singular!\n", dynl_error());
-      killhdl2(pl,&(basePack->idroot),NULL); // remove package
+      errorreported=0;
+      if(IDPACKAGE(pl)->idroot==NULL)
+        killhdl2(pl,&(basePack->idroot),NULL); // remove package
     }
   }
 
   load_modules_end:
   return RET;
-#endif /*STATIC */
+}
+
+BOOLEAN load_modules(const char *newlib, char *fullname, BOOLEAN autoexport)
+{
+  GLOBAL_VAR static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
+  pthread_mutex_lock(&mutex);
+  BOOLEAN r = load_modules_aux(newlib, fullname, autoexport);
+  pthread_mutex_unlock(&mutex);
+  return r;
 }
 #endif /* HAVE_DYNAMIC_LOADING */
 /*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
@@ -1108,6 +1317,7 @@ BOOLEAN load_builtin(const char *newlib, BOOLEAN autoexport, SModulFunc_t init)
     pl = enterid( plib,0, PACKAGE_CMD, &IDROOT, TRUE );
     IDPACKAGE(pl)->libname=omStrDup(newlib);
   }
+  omFree(plib);
   IDPACKAGE(pl)->language = LANG_C;
 
   IDPACKAGE(pl)->handle=(void *)NULL;
@@ -1139,7 +1349,7 @@ void module_help_main(const char *newlib,const char *help)
   {
     package s=currPack;
     currPack=IDPACKAGE(pl);
-    idhdl h=enterid(omStrDup("info"),0,STRING_CMD,&IDROOT,FALSE);
+    idhdl h=enterid("info",0,STRING_CMD,&IDROOT,FALSE);
     IDSTRING(h)=omStrDup(help);
     currPack=s;
   }
@@ -1158,7 +1368,7 @@ void module_help_proc(const char *newlib,const char *p, const char *help)
     buff[255]='\0';
     strncpy(buff,p,255);
     strncat(buff,"_help",255-strlen(p));
-    idhdl h=enterid(omStrDup(buff),0,STRING_CMD,&IDROOT,FALSE);
+    idhdl h=enterid(buff,0,STRING_CMD,&IDROOT,FALSE);
     IDSTRING(h)=omStrDup(help);
     currPack=s;
   }
@@ -1216,10 +1426,12 @@ char *iiConvName(const char *libname)
   char *tmpname = omStrDup(libname);
   char *p = strrchr(tmpname, DIR_SEP);
   char *r;
-  if(p==NULL) p = tmpname;
-  else p++;
-  r = (char *)strchr(p, '.');
-  if( r!= NULL) *r = '\0';
+  if(p==NULL) p = tmpname; else p++;
+  // p is now the start of the file name (without path)
+  r=p;
+  while(isalnum(*r)||(*r=='_')) r++;
+  // r point the the end of the main part of the filename
+  *r = '\0';
   r = omStrDup(p);
   *r = mytoupper(*r);
   // printf("iiConvName: '%s' '%s' => '%s'\n", libname, tmpname, r);
@@ -1294,7 +1506,7 @@ void libstack::push(const char */*p*/, char *libn)
       ls->next = this;
       ls->libname = omStrDup(libn);
       ls->to_be_done = TRUE;
-      if(this != NULL) ls->cnt = this->cnt+1; else ls->cnt = 0;
+      if(library_stack != NULL) ls->cnt = library_stack->cnt+1; else ls->cnt = 0;
       library_stack = ls;
     }
   }
