@@ -7,68 +7,60 @@
  ***************************************************************/
 #define TRANSEXT_PRIVATES 1 /* allow access to transext internals */
 
-#include <kernel/mod2.h>
+#include "kernel/mod2.h"
 
-#include <omalloc/omalloc.h>
+#include "misc/intvec.h"
+#include "misc/options.h"
 
-#include <misc/intvec.h>
-#include <misc/options.h>
+#include "reporter/si_signals.h"
+#include "reporter/s_buff.h"
 
-#include <reporter/si_signals.h>
-#include <reporter/s_buff.h>
+#include "coeffs/bigintmat.h"
+#include "coeffs/longrat.h"
 
-#include <coeffs/bigintmat.h>
-#include <coeffs/longrat.h>
+#include "polys/monomials/ring.h"
+#include "polys/monomials/p_polys.h"
+#include "polys/ext_fields/transext.h"
+#include "polys/simpleideals.h"
+#include "polys/matpol.h"
 
-#include <polys/monomials/ring.h>
-#include <polys/monomials/p_polys.h>
-#include <polys/ext_fields/transext.h>
-#include <polys/simpleideals.h>
-#include <polys/matpol.h>
+#include "kernel/oswrapper/timer.h"
+#include "kernel/oswrapper/feread.h"
+#include "kernel/oswrapper/rlimit.h"
 
-#include <kernel/oswrapper/timer.h>
-#include <kernel/oswrapper/feread.h>
-#include <kernel/oswrapper/rlimit.h>
-
-#include <Singular/tok.h>
-#include <Singular/ipid.h>
-#include <Singular/ipshell.h>
-#include <Singular/subexpr.h>
-#include <Singular/links/silink.h>
-#include <Singular/cntrlc.h>
-#include <Singular/lists.h>
-#include <Singular/blackbox.h>
-#include <Singular/links/ssiLink.h>
+#include "Singular/tok.h"
+#include "Singular/ipid.h"
+#include "Singular/ipshell.h"
+#include "Singular/subexpr.h"
+#include "Singular/links/silink.h"
+#include "Singular/cntrlc.h"
+#include "Singular/feOpt.h"
+#include "Singular/lists.h"
+#include "Singular/blackbox.h"
+#include "Singular/links/ssiLink.h"
 
 #ifdef HAVE_SIMPLEIPC
-#include <Singular/links/simpleipc.h>
+#include "Singular/links/simpleipc.h"
 #endif
 
-#include <stdio.h>
-#include <fcntl.h>
 #include <errno.h>
-#include <unistd.h>
-#include <sys/types.h>
-#include <signal.h>
 #include <sys/types.h>          /* for portability */
-#include <sys/select.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
 #include <ctype.h>   /*for isdigit*/
 #include <netdb.h>
-#include <sys/wait.h>
-#include <time.h>
+#include <netinet/in.h> /* for htons etc.*/
 
-#define SSI_VERSION 10
+#define SSI_VERSION 13
 // 5->6: changed newstruct representation
 // 6->7: attributes
 // 7->8: qring
 // 8->9: module: added rank
 // 9->10: tokens in grammar.h/tok.h reorganized
 // 10->11: extended ring descr. for named coeffs (not in used until 4.1)
+// 11->12: add rank to ideal/module, add smatrix
+// 12->13: NC rings
 
-link_list ssiToBeClosed=NULL;
-volatile BOOLEAN ssiToBeClosed_inactive=TRUE;
+VAR link_list ssiToBeClosed=NULL;
+VAR volatile BOOLEAN ssiToBeClosed_inactive=TRUE;
 
 // forward declarations:
 void ssiWritePoly_R(const ssiInfo *d, int typ, poly p, const ring r);
@@ -83,25 +75,75 @@ BOOLEAN ssiSetCurrRing(const ring r) /* returned: not accepted */
   //  Print("need to change the ring, currRing:%s, switch to: ssiRing%d\n",IDID(currRingHdl),nr);
   //  else
   //  Print("no ring, switch to ssiRing%d\n",nr);
-  if (!rEqual(r,currRing,1))
+  if (r==currRing)
+  {
+    r->ref++;
+    currRingHdl=rFindHdl(r,currRingHdl);
+    return TRUE;
+  }
+  else if ((currRing==NULL) || (!rEqual(r,currRing,1)))
   {
     char name[20];
     int nr=0;
-    do
-    { sprintf(name,"ssiRing%d",nr); nr++; }
-    while(IDROOT->get(name, 0)!=NULL);
-    idhdl h=enterid(omStrDup(name),0,RING_CMD,&IDROOT,FALSE);
-    IDRING(h)=r;
-    r->ref++;
+    idhdl h=NULL;
+    loop
+    {
+      sprintf(name,"ssiRing%d",nr); nr++;
+      h=IDROOT->get(name, 0);
+      if (h==NULL)
+      {
+        h=enterid(name,0,RING_CMD,&IDROOT,FALSE);
+        IDRING(h)=r;
+        r->ref=2; /*d->r and h */
+        break;
+      }
+      else if ((IDTYP(h)==RING_CMD)
+      && (rEqual(r,IDRING(h),1)))
+      {
+        IDRING(h)->ref++;
+        break;
+      }
+    }
     rSetHdl(h);
     return FALSE;
   }
   else
   {
     rKill(r);
+    currRing->ref++;
     return TRUE;
   }
 }
+void ssiCheckCurrRing(const ring r)
+{
+  if ((r!=currRing)
+  ||(currRingHdl==NULL)
+  ||(IDRING(currRingHdl)!=r))
+  {
+    char name[20];
+    int nr=0;
+    idhdl h=NULL;
+    loop
+    {
+      sprintf(name,"ssiRing%d",nr); nr++;
+      h=IDROOT->get(name, 0);
+      if (h==NULL)
+      {
+        h=enterid(name,0,RING_CMD,&IDROOT,FALSE);
+        IDRING(h)=r;
+        r->ref=2; /*d->r and h */
+        break;
+      }
+      else if ((IDTYP(h)==RING_CMD)
+      && (rEqual(r,IDRING(h),1)))
+      {
+        break;
+      }
+    }
+    rSetHdl(h);
+  }
+  assume((currRing==r) || rEqual(r,currRing));
+}
 // the implementation of the functions:
 void ssiWriteInt(const ssiInfo *d,const int i)
 {
@@ -117,7 +159,7 @@ void ssiWriteString(const ssiInfo *d,const char *s)
 
 void ssiWriteBigInt(const ssiInfo *d, const number n)
 {
- n_WriteFd(n,d->f_write,coeffs_BIGINT);
+ n_WriteFd(n,d,coeffs_BIGINT);
 }
 
 void ssiWriteNumber_CF(const ssiInfo *d, const number n, const coeffs cf)
@@ -143,7 +185,7 @@ void ssiWriteNumber_CF(const ssiInfo *d, const number n, const coeffs cf)
   }
   else if (cf->cfWriteFd!=NULL)
   {
-    n_WriteFd(n,d->f_write,cf);
+    n_WriteFd(n,d,cf);
   }
   else WerrorS("coeff field not implemented");
 }
@@ -226,12 +268,33 @@ void ssiWriteRing_R(ssiInfo *d,const ring r)
     }
     else
     {
-      fprintf(d->f_write,"0 "/*ideal with 0 entries */);
+      fputs("0 ",d->f_write/*ideal with 0 entries */);
     }
   }
   else /* dummy ring r==NULL*/
   {
-    fprintf(d->f_write,"0 0 0 0 "/*,r->ch,r->N, blocks, q-ideal*/);
+    fputs("0 0 0 0 "/*,r->ch,r->N, blocks, q-ideal*/,d->f_write);
+  }
+  if (rIsLPRing(r)) // cannot be combined with 23 2
+  {
+    fprintf(d->f_write,"23 1 %d %d ",SI_LOG2(r->bitmask),r->isLPring);
+  }
+  else
+  {
+    unsigned long bm=0;
+    int b=0;
+    bm=rGetExpSize(bm,b,r->N);
+    if (r->bitmask!=bm)
+    {
+      fprintf(d->f_write,"23 0 %d ",SI_LOG2(r->bitmask));
+    }
+    if (rIsPluralRing(r))
+    {
+      fputs("23 2 ",d->f_write);
+      void ssiWriteIdeal(const ssiInfo *d, int typ,const ideal I);
+      ssiWriteIdeal(d,MATRIX_CMD,(ideal)r->GetNC()->C);
+      ssiWriteIdeal(d,MATRIX_CMD,(ideal)r->GetNC()->D);
+    }
   }
 }
 
@@ -241,9 +304,10 @@ void ssiWriteRing(ssiInfo *d,const ring r)
   /* ch=-1: transext, coeff ring follows */
   /* ch=-2: algext, coeff ring and minpoly follows */
   /* ch=-3: cf name follows */
+  /* ch=-4: NULL */
   if ((r==NULL)||(r->cf==NULL))
   {
-    WerrorS("undefined ring");
+    fputs("-4 ",d->f_write);
     return;
   }
   if (r==currRing) // see recursive calls for transExt/algExt
@@ -282,8 +346,9 @@ void ssiWritePoly(const ssiInfo *d, int typ, poly p)
 
 void ssiWriteIdeal_R(const ssiInfo *d, int typ,const ideal I, const ring R)
 {
-   // syntax: 7 # of elements <poly 1> <poly2>.....
-   // syntax: 8 <rows> <cols> <poly 1> <poly2>.....
+   // syntax: 7 # of elements <poly 1> <poly2>.....(ideal,module,smatrix)
+   // syntax: 8 <rows> <cols> <poly 1> <poly2>.....(matrix)
+   // syntax
    matrix M=(matrix)I;
    int mn;
    if (typ==MATRIX_CMD)
@@ -299,8 +364,10 @@ void ssiWriteIdeal_R(const ssiInfo *d, int typ,const ideal I, const ring R)
 
    int i;
    int tt;
-   if (typ==MODUL_CMD) tt=VECTOR_CMD;
-   else                tt=POLY_CMD;
+   if ((typ==MODUL_CMD)||(typ==SMATRIX_CMD))
+     tt=VECTOR_CMD;
+   else
+     tt=POLY_CMD;
 
    for(i=0;i<mn;i++)
    {
@@ -397,7 +464,7 @@ number ssiReadNumber_CF(const ssiInfo *d, const coeffs cf)
 {
   if (cf->cfReadFd!=NULL)
   {
-     return n_ReadFd(d->f_read,cf);
+     return n_ReadFd(d,cf);
   }
   else if (getCoeffType(cf) == n_transExt)
   {
@@ -427,7 +494,7 @@ number ssiReadBigInt(const ssiInfo *d)
   return n;
 }
 
-number ssiReadNumber(const ssiInfo *d)
+number ssiReadNumber(ssiInfo *d)
 {
   return ssiReadNumber_CF(d,d->r->cf);
 }
@@ -435,10 +502,12 @@ number ssiReadNumber(const ssiInfo *d)
 ring ssiReadRing(const ssiInfo *d)
 {
 /* syntax is <ch> <N> <l1> <v1> ...<lN> <vN> <number of orderings> <ord1> <block0_1> <block1_1> .... <Q-ideal> */
-  int ch, N,i;
-  char **names;
+  int ch;
   ch=s_readint(d->f_read);
-  N=s_readint(d->f_read);
+  if (ch==-4)
+    return NULL;
+  int N=s_readint(d->f_read);
+  char **names;
   coeffs cf=NULL;
   if (ch==-3)
   {
@@ -454,7 +523,7 @@ ring ssiReadRing(const ssiInfo *d)
   if (N!=0)
   {
     names=(char**)omAlloc(N*sizeof(char*));
-    for(i=0;i<N;i++)
+    for(int i=0;i<N;i++)
     {
       names[i]=ssiReadString(d);
     }
@@ -462,13 +531,13 @@ ring ssiReadRing(const ssiInfo *d)
   // read the orderings:
   int num_ord; // number of orderings
   num_ord=s_readint(d->f_read);
-  int *ord=(int *)omAlloc0((num_ord+1)*sizeof(int));
+  rRingOrder_t *ord=(rRingOrder_t *)omAlloc0((num_ord+1)*sizeof(rRingOrder_t));
   int *block0=(int *)omAlloc0((num_ord+1)*sizeof(int));
   int *block1=(int *)omAlloc0((num_ord+1)*sizeof(int));
   int **wvhdl=(int**)omAlloc0((num_ord+1)*sizeof(int*));
-  for(i=0;i<num_ord;i++)
+  for(int i=0;i<num_ord;i++)
   {
-    ord[i]=s_readint(d->f_read);
+    ord[i]=(rRingOrder_t)s_readint(d->f_read);
     block0[i]=s_readint(d->f_read);
     block1[i]=s_readint(d->f_read);
     switch(ord[i])
@@ -533,7 +602,7 @@ ring ssiReadRing(const ssiInfo *d)
     else
     {
       Werror("ssi: read unknown coeffs type (%d)",ch);
-      for(i=0;i<N;i++)
+      for(int i=0;i<N;i++)
       {
         omFree(names[i]);
       }
@@ -543,20 +612,21 @@ ring ssiReadRing(const ssiInfo *d)
     ideal q=ssiReadIdeal_R(d,r);
     if (IDELEMS(q)==0) omFreeBin(q,sip_sideal_bin);
     else r->qideal=q;
-    for(i=0;i<N;i++)
+    for(int i=0;i<N;i++)
     {
       omFree(names[i]);
     }
     omFreeSize(names,N*sizeof(char*));
+    r->ref=1;
     return r;
   }
 }
 
-poly ssiReadPoly_R(const ssiInfo *D, const ring r)
+poly ssiReadPoly_R(const ssiInfo *d, const ring r)
 {
 // < # of terms> < term1> < .....
   int n,i,l;
-  n=ssiReadInt(D->f_read); // # of terms
+  n=ssiReadInt(d->f_read); // # of terms
   //Print("poly: terms:%d\n",n);
   poly p;
   poly ret=NULL;
@@ -564,15 +634,15 @@ poly ssiReadPoly_R(const ssiInfo *D, const ring r)
   for(l=0;l<n;l++) // read n terms
   {
 // coef,comp.exp1,..exp N
-    p=p_Init(r);
-    pSetCoeff0(p,ssiReadNumber_CF(D,r->cf));
-    int d;
-    d=s_readint(D->f_read);
-    p_SetComp(p,d,r);
+    p=p_Init(r,r->PolyBin);
+    pSetCoeff0(p,ssiReadNumber_CF(d,r->cf));
+    int D;
+    D=s_readint(d->f_read);
+    p_SetComp(p,D,r);
     for(i=1;i<=rVar(r);i++)
     {
-      d=s_readint(D->f_read);
-      p_SetExp(p,i,d,r);
+      D=s_readint(d->f_read);
+      p_SetExp(p,i,D,r);
     }
     p_Setm(p,r);
     p_Test(p,r);
@@ -583,18 +653,18 @@ poly ssiReadPoly_R(const ssiInfo *D, const ring r)
  return ret;
 }
 
-poly ssiReadPoly(const ssiInfo *D)
+poly ssiReadPoly(ssiInfo *d)
 {
-// < # of terms> < term1> < .....
-  return ssiReadPoly_R(D,D->r);
+  return ssiReadPoly_R(d,d->r);
 }
 
 ideal ssiReadIdeal_R(const ssiInfo *d,const ring r)
 {
+// < # of terms> < term1> < .....
   int n,i;
   ideal I;
   n=s_readint(d->f_read);
-  I=idInit(n,1);
+  I=idInit(n,1); // will be fixed later for module/smatrix
   for(i=0;i<IDELEMS(I);i++) // read n terms
   {
     I->m [i]=ssiReadPoly_R(d,r);
@@ -602,12 +672,12 @@ ideal ssiReadIdeal_R(const ssiInfo *d,const ring r)
   return I;
 }
 
-ideal ssiReadIdeal(const ssiInfo *d)
+ideal ssiReadIdeal(ssiInfo *d)
 {
   return ssiReadIdeal_R(d,d->r);
 }
 
-matrix ssiReadMatrix(const ssiInfo *d)
+matrix ssiReadMatrix(ssiInfo *d)
 {
   int n,m;
   m=s_readint(d->f_read);
@@ -742,9 +812,17 @@ void ssiReadBlackbox(leftv res, si_link l)
   blackboxIsCmd(name,tok);
   if (tok>MAX_TOK)
   {
+    ring save_ring=currRing;
+    idhdl save_hdl=currRingHdl;
     blackbox *b=getBlackboxStuff(tok);
     res->rtyp=tok;
     b->blackbox_deserialize(&b,&(res->data),l);
+    if (save_ring!=currRing)
+    {
+      rChangeCurrRing(save_ring);
+      if (save_hdl!=NULL) rSetHdl(save_hdl);
+      else currRingHdl=NULL;
+    }
   }
   else
   {
@@ -773,6 +851,45 @@ void ssiReadAttrib(leftv res, si_link l)
   }
   res->flag=fl;
 }
+void ssiReadRingProperties(si_link l)
+{
+  ssiInfo *d=(ssiInfo*)l->data;
+  int what=s_readint(d->f_read);
+  switch(what)
+  {
+    case 0: // bitmask
+    {
+      int lb=s_readint(d->f_read);
+      unsigned long bm=~0L;
+      bm=bm<<lb;
+      bm=~bm;
+      rUnComplete(d->r);
+      d->r->bitmask=bm;
+      rComplete(d->r);
+      break;
+    }
+    case 1: // LPRing
+    {
+      int lb=s_readint(d->f_read);
+      int isLPring=s_readint(d->f_read);
+      unsigned long bm=~0L;
+      bm=bm<<lb;
+      bm=~bm;
+      rUnComplete(d->r);
+      d->r->bitmask=bm;
+      d->r->isLPring=isLPring;
+      rComplete(d->r);
+      break;
+    }
+    case 2: // Plural rings
+    {
+      matrix C=ssiReadMatrix(d);
+      matrix D=ssiReadMatrix(d);
+      nc_CallPlural(C,D,NULL,NULL,d->r,true,true,false,d->r,false);
+      break;
+    }
+  }
+}
 //**************************************************************************/
 
 BOOLEAN ssiOpen(si_link l, short flag, leftv u)
@@ -833,6 +950,8 @@ BOOLEAN ssiOpen(si_link l, short flag, leftv u)
           sigemptyset(&sigint);
           sigaddset(&sigint, SIGINT);
           sigprocmask(SIG_BLOCK, &sigint, NULL);
+          /* set #cpu to 1 for the child:*/
+	  feSetOptValue(FE_OPT_CPUS,1);
 
           link_list hh=(link_list)ssiToBeClosed->next;
           /* we know: l is the first entry in ssiToBeClosed-list */
@@ -1270,6 +1389,8 @@ leftv ssiRead1(si_link l)
            res->data=(char *)ssiReadString(d);
            break;
     case 3:res->rtyp=NUMBER_CMD;
+           if (d->r==NULL) goto no_ring;
+           ssiCheckCurrRing(d->r);
            res->data=(char *)ssiReadNumber(d);
            break;
     case 4:res->rtyp=BIGINT_CMD;
@@ -1278,16 +1399,13 @@ leftv ssiRead1(si_link l)
     case 15:
     case 5:{
              d->r=ssiReadRing(d);
-             if (d->r==NULL) return NULL;
+             if (errorreported) return NULL;
              res->data=(char*)d->r;
+             if (d->r!=NULL) d->r->ref++;
              res->rtyp=RING_CMD;
-             // we are in the top-level, so set the basering to d->r:
-             if (d->r!=NULL)
-             {
-               if(ssiSetCurrRing(d->r)) { d->r=currRing; d->r->ref++; }
-             }
              if (t==15) // setring
              {
+               if(ssiSetCurrRing(d->r)) { d->r=currRing; }
                omFreeBin(res,sleftv_bin);
                return ssiRead1(l);
              }
@@ -1295,22 +1413,29 @@ leftv ssiRead1(si_link l)
            break;
     case 6:res->rtyp=POLY_CMD;
            if (d->r==NULL) goto no_ring;
+           ssiCheckCurrRing(d->r);
            res->data=(char*)ssiReadPoly(d);
            break;
     case 7:res->rtyp=IDEAL_CMD;
            if (d->r==NULL) goto no_ring;
+           ssiCheckCurrRing(d->r);
            res->data=(char*)ssiReadIdeal(d);
            break;
     case 8:res->rtyp=MATRIX_CMD;
            if (d->r==NULL) goto no_ring;
+           ssiCheckCurrRing(d->r);
            res->data=(char*)ssiReadMatrix(d);
            break;
     case 9:res->rtyp=VECTOR_CMD;
            if (d->r==NULL) goto no_ring;
+           ssiCheckCurrRing(d->r);
            res->data=(char*)ssiReadPoly(d);
            break;
-    case 10:res->rtyp=MODUL_CMD;
+    case 10:
+    case 22:if (t==22) res->rtyp=SMATRIX_CMD;
+           else        res->rtyp=MODUL_CMD;
            if (d->r==NULL) goto no_ring;
+           ssiCheckCurrRing(d->r);
            {
              int rk=s_readint(d->f_read);
              ideal M=ssiReadIdeal(d);
@@ -1355,6 +1480,9 @@ leftv ssiRead1(si_link l)
              break;
     case 21: ssiReadAttrib(res,l);
              break;
+    case 23: ssiReadRingProperties(l);
+             return ssiRead1(l);
+             break;
     // ------------
     case 98: // version
              {
@@ -1396,7 +1524,7 @@ leftv ssiRead1(si_link l)
   && (currRing!=d->r)
   && (res->RingDependend()))
   {
-    if(ssiSetCurrRing(d->r)) { d->r=currRing; d->r->ref++; }
+    if(ssiSetCurrRing(d->r)) { d->r=currRing; }
   }
   return res;
 no_ring: WerrorS("no ring");
@@ -1435,14 +1563,16 @@ BOOLEAN ssiWrite(si_link l, leftv data)
     int tt=data->Typ();
     void *dd=data->Data();
     attr *aa=data->Attribute();
-    BOOLEAN with_attr=FALSE;
-    if (((*aa)!=NULL)||(data->flag!=0))
+    if ((aa!=NULL) && ((*aa)!=NULL)) // n user attributes
     {
       attr a=*aa;
       int n=0;
       while(a!=NULL) { n++; a=a->next;}
       fprintf(d->f_write,"21 %d %d ",data->flag,n);
-      a=*aa;
+    }
+    else if (data->flag!=0) // only "flag" attributes
+    {
+      fprintf(d->f_write,"21 %d 0 ",data->flag);
     }
     if ((dd==NULL) && (data->name!=NULL) && (tt==0)) tt=DEF_CMD;
       // return pure undefined names as def
@@ -1474,6 +1604,19 @@ BOOLEAN ssiWrite(si_link l, leftv data)
           case RING_CMD:fputs("5 ",d->f_write);
                         ssiWriteRing(d,(ring)dd);
                         break;
+          case BUCKET_CMD:
+                        {
+                          sBucket_pt b=(sBucket_pt)dd;
+                          if (d->r!=sBucketGetRing(b))
+                          {
+                            fputs("15 ",d->f_write);
+                            ssiWriteRing(d,sBucketGetRing(b));
+                            if (d->level<=1) fputc('\n',d->f_write);
+                          }
+                          fputs("6 ",d->f_write);
+                          ssiWritePoly(d,tt,sBucketPeek(b));
+                          break;
+                        }
           case POLY_CMD:
           case VECTOR_CMD:
                         if (d->r!=currRing)
@@ -1489,6 +1632,7 @@ BOOLEAN ssiWrite(si_link l, leftv data)
           case IDEAL_CMD:
           case MODUL_CMD:
           case MATRIX_CMD:
+          case SMATRIX_CMD:
                         if (d->r!=currRing)
                         {
                           fputs("15 ",d->f_write);
@@ -1497,7 +1641,8 @@ BOOLEAN ssiWrite(si_link l, leftv data)
                         }
                         if(tt==IDEAL_CMD)       fputs("7 ",d->f_write);
                         else if(tt==MATRIX_CMD) fputs("8 ",d->f_write);
-                        else
+                        else if(tt==SMATRIX_CMD) fputs("22 ",d->f_write);
+                        else /* tt==MODUL_CMD*/
                         {
                           ideal M=(ideal)dd;
                           fprintf(d->f_write,"10 %d ",(int)M->rank);
@@ -1813,7 +1958,7 @@ int ssiBatch(const char *host, const char * port)
   if (slOpen(l,SI_LINK_OPEN,NULL)) return 1;
   SI_LINK_SET_RW_OPEN_P(l);
 
-  idhdl id = enterid(omStrDup("link_ll"), 0, LINK_CMD, &IDROOT, FALSE);
+  idhdl id = enterid("link_ll", 0, LINK_CMD, &IDROOT, FALSE);
   IDLINK(id) = l;
 
   loop
@@ -1833,10 +1978,10 @@ int ssiBatch(const char *host, const char * port)
   exit(0);
 }
 
-static int ssiReserved_P=0;
-static int ssiReserved_sockfd;
-static  struct sockaddr_in ssiResverd_serv_addr;
-static int  ssiReserved_Clients;
+STATIC_VAR int ssiReserved_P=0;
+STATIC_VAR int ssiReserved_sockfd;
+STATIC_VAR struct sockaddr_in ssiResverd_serv_addr;
+STATIC_VAR int  ssiReserved_Clients;
 int ssiReservePort(int clients)
 {
   if (ssiReserved_P!=0)
@@ -1872,7 +2017,7 @@ int ssiReservePort(int clients)
   return portno;
 }
 
-extern si_link_extension si_link_root;
+EXTERN_VAR si_link_extension si_link_root;
 si_link ssiCommandLink()
 {
   if (ssiReserved_P==0)
@@ -2003,6 +2148,17 @@ static BOOLEAN DumpSsiIdhdl(si_link l, idhdl h)
   if ((type_id == RING_CMD) && (strncmp(IDID(h),"ssiRing",7)==0))
     return FALSE;
 
+  // do not dump default cring:
+  if (type_id == CRING_CMD)
+  {
+    if (strcmp(IDID(h),"ZZ")==0) return FALSE;
+    if (strcmp(IDID(h),"QQ")==0) return FALSE;
+    #ifdef SINGULAR_4_2
+    if (strcmp(IDID(h),"AE")==0) return FALSE;
+    if (strcmp(IDID(h),"QAE")==0) return FALSE;
+    #endif
+  }
+
   command D=(command)omAlloc0(sizeof(*D));
   sleftv tmp;
   memset(&tmp,0,sizeof(tmp));
@@ -2133,6 +2289,10 @@ BOOLEAN ssiGetDump(si_link l)
 // 19 bigintmat <r> <c> ...
 // 20 blackbox <name> 1 <len> ...
 // 21 attrib <bit-attrib> <len> <a-name1> <val1>... <data>
+// 22 smatrix
+// 23 0 <log(bitmask)> ring properties: max.exp.
+// 23 1 <log(bitmask)> <r->IsLPRing> ring properties:LPRing
+// 23 2 <matrix C> <matrix D> ring properties: PLuralRing
 //
 // 98: verify version: <ssi-version> <MAX_TOK> <OPT1> <OPT2>
 // 99: quit Singular
