@@ -13,27 +13,24 @@
 // include header files
 #define PLURAL_INTERNAL_DECLARATIONS 1
 
-#include <kernel/mod2.h>
-#include <misc/sirandom.h>
-
-#include <reporter/si_signals.h>
-
-#include <factory/factory.h>
-
-#include <coeffs/si_gmp.h>
-#include <coeffs/coeffs.h>
-#include <coeffs/OPAE.h>
-#include <coeffs/OPAEQ.h>
-#include <coeffs/OPAEp.h>
-#include <coeffs/flintcf_Q.h>
-#include <coeffs/flintcf_Zn.h>
-
-#include <polys/ext_fields/algext.h>
-#include <polys/ext_fields/transext.h>
-#include <polys/nc/gb_hack.h>
+#include "kernel/mod2.h"
+#include "misc/sirandom.h"
+#include "omalloc/omalloc.h"
+#include "misc/mylimits.h"
+#include "reporter/si_signals.h"
+#include "factory/factory.h"
+#include "coeffs/si_gmp.h"
+#include "coeffs/coeffs.h"
+#include "coeffs/flintcf_Q.h"
+#include "coeffs/flintcf_Qrat.h"
+#include "coeffs/flintcf_Zn.h"
+#include "coeffs/rmodulon.h"
+#include "polys/ext_fields/algext.h"
+#include "polys/ext_fields/transext.h"
+#include "polys/nc/gb_hack.h"
 
 #ifdef HAVE_SIMPLEIPC
-#include <Singular/links/simpleipc.h>
+#include "Singular/links/simpleipc.h"
 #endif
 
 #include "misc_ip.h"
@@ -41,7 +38,42 @@
 #include "feOpt.h"
 #include "links/silink.h"
 #include "mod_lib.h"
-#include <Singular/distrib.h>
+#include "Singular/distrib.h"
+
+#include "misc/options.h"
+#include "misc/intvec.h"
+
+#include "polys/monomials/ring.h"
+#include "polys/templates/p_Procs.h"
+
+#include "kernel/GBEngine/kstd1.h"
+#include "kernel/oswrapper/timer.h"
+#include "resources/feResource.h"
+#include "kernel/oswrapper/feread.h"
+
+#include "subexpr.h"
+#include "cntrlc.h"
+#include "ipshell.h"
+
+#include "fehelp.h"
+
+#ifdef HAVE_READLINE
+  #ifdef READLINE_READLINE_H_OK
+    #include <readline/readline.h>
+  #endif
+  #ifndef RL_VERSION_MAJOR
+    #define RL_VERSION_MAJOR 0
+  #endif
+#endif
+
+#ifdef HAVE_NTL
+#include <NTL/version.h>
+#include <NTL/tools.h>
+#ifdef NTL_CLIENT
+NTL_CLIENT
+#endif
+#endif
+
 
 static FORCE_INLINE void number2mpz(number n, mpz_t m){ number2mpz(n, coeffs_BIGINT, m); }
 static FORCE_INLINE number mpz2number(mpz_t m){ return mpz2number(m, coeffs_BIGINT); }
@@ -80,7 +112,7 @@ void setListEntry_ui(lists L, int index, unsigned long ui)
 }
 
 /* Factoring with Pollard's rho method. stolen from GMP/demos */
-static unsigned add[] = {4, 2, 4, 2, 4, 6, 2, 6};
+STATIC_VAR unsigned add[] = {4, 2, 4, 2, 4, 6, 2, 6};
 
 static int factor_using_division (mpz_t t, unsigned int limit,lists primes, int *multiplicities,int &index, unsigned long bound)
 {
@@ -106,7 +138,7 @@ static int factor_using_division (mpz_t t, unsigned int limit,lists primes, int
   loop
   {
     mpz_tdiv_qr_ui (q, r, t, 3);
-    if (mpz_cmp_ui (r, 0) != 0)
+    if (mpz_sgn1 (r) != 0)
         break;
     mpz_set (t, q);
     f++;
@@ -120,7 +152,7 @@ static int factor_using_division (mpz_t t, unsigned int limit,lists primes, int
   loop
   {
     mpz_tdiv_qr_ui (q, r, t, 5);
-    if (mpz_cmp_ui (r, 0) != 0)
+    if (mpz_sgn1 (r) != 0)
         break;
     mpz_set (t, q);
     f++;
@@ -138,7 +170,7 @@ static int factor_using_division (mpz_t t, unsigned int limit,lists primes, int
   while (mpz_cmp_ui (t, 1) != 0)
   {
     mpz_tdiv_qr_ui (q, r, t, f);
-    if (mpz_cmp_ui (r, 0) != 0)
+    if (mpz_sgn1 (r) != 0)
     {
       f += addv[ai];
       if (mpz_cmp_ui (t, f) < 0)
@@ -186,10 +218,10 @@ static void factor_using_pollard_rho (mpz_t n, unsigned long a, lists primes, in
 
   mpz_init (t1);
   mpz_init (t2);
-  mpz_init_set_si (last_f, 0);
-  mpz_init_set_si (y, 2);
-  mpz_init_set_si (x, 2);
-  mpz_init_set_si (x1, 2);
+  mpz_init_set_ui (last_f, 0);
+  mpz_init_set_ui (y, 2);
+  mpz_init_set_ui (x, 2);
+  mpz_init_set_ui (x1, 2);
   mpz_init_set_ui (P, 1);
   k = 1;
   l = 1;
@@ -381,27 +413,6 @@ lists primeFactorisation(const number n, const int pBound)
   return L;
 }
 
-#include <omalloc/omalloc.h>
-#include <misc/mylimits.h>
-
-#include <misc/options.h>
-#include <misc/intvec.h>
-
-#include <polys/monomials/ring.h>
-#include <polys/templates/p_Procs.h>
-
-#include <kernel/GBEngine/kstd1.h>
-#include <kernel/oswrapper/timer.h>
-#include <resources/feResource.h>
-#include <kernel/oswrapper/feread.h>
-
-#include "subexpr.h"
-#include "cntrlc.h"
-#include "ipid.h"
-#include "ipshell.h"
-
-#include "fehelp.h"
-
 #ifdef HAVE_STATIC
 #undef HAVE_DYN_RL
 #endif
@@ -428,14 +439,6 @@ lists primeFactorisation(const number n, const int pBound)
 *}
 */
 
-#include <string.h>
-#include <unistd.h>
-#include <stdio.h>
-#include <stddef.h>
-#include <stdlib.h>
-#include <time.h>
-
-
 void singular_example(char *str)
 {
   assume(str!=NULL);
@@ -448,7 +451,7 @@ void singular_example(char *str)
     *ss='\0';
     ss--;
   }
-  idhdl h=IDROOT->get(s,myynest);
+  idhdl h=IDROOT->get_level(s,0);
   if ((h!=NULL) && (IDTYP(h)==PROC_CMD))
   {
     char *lib=iiGetLibName(IDPROC(h));
@@ -523,13 +526,14 @@ const struct soptionStruct optionStruct[]=
   {"sugarCrit",    Sy_bit(OPT_SUGARCRIT),      ~Sy_bit(OPT_SUGARCRIT)   },
   {"teach",        Sy_bit(OPT_DEBUG),          ~Sy_bit(OPT_DEBUG)  },
   {"notSyzMinim",  Sy_bit(OPT_NO_SYZ_MINIM),   ~Sy_bit(OPT_NO_SYZ_MINIM)  },
-  /* 9 return SB in syz, quotient, intersect */
+  /* 9 return SB in syz, quotient, intersect, modulo */
   {"returnSB",     Sy_bit(OPT_RETURN_SB),      ~Sy_bit(OPT_RETURN_SB)  },
   {"fastHC",       Sy_bit(OPT_FASTHC),         ~Sy_bit(OPT_FASTHC)  },
   /* 11-19 sort in L/T */
   {"staircaseBound",Sy_bit(OPT_STAIRCASEBOUND),~Sy_bit(OPT_STAIRCASEBOUND)  },
   {"multBound",    Sy_bit(OPT_MULTBOUND),      ~Sy_bit(OPT_MULTBOUND)  },
   {"degBound",     Sy_bit(OPT_DEGBOUND),       ~Sy_bit(OPT_DEGBOUND)  },
+  {"redTailSyz",   Sy_bit(OPT_REDTAIL_SYZ),    ~Sy_bit(OPT_REDTAIL_SYZ) },
   /* 25 no redTail(p)/redTail(s) */
   {"redTail",      Sy_bit(OPT_REDTAIL),        ~Sy_bit(OPT_REDTAIL)  },
   {"redThrough",   Sy_bit(OPT_REDTHROUGH),     ~Sy_bit(OPT_REDTHROUGH)  },
@@ -545,6 +549,7 @@ const struct soptionStruct optionStruct[]=
 
 const struct soptionStruct verboseStruct[]=
 {
+  {"assign_none",Sy_bit(V_ASSIGN_NONE),~Sy_bit(V_ASSIGN_NONE)},
   {"mem",      Sy_bit(V_SHOW_MEM),  ~Sy_bit(V_SHOW_MEM)   },
   {"yacc",     Sy_bit(V_YACC),      ~Sy_bit(V_YACC)       },
   {"redefine", Sy_bit(V_REDEFINE),  ~Sy_bit(V_REDEFINE)   },
@@ -645,7 +650,7 @@ BOOLEAN setOption(leftv res, leftv v)
             si_opt_1 &= ~Sy_bit(OPT_REDTHROUGH);
         }
         else
-          Warn("cannot set option");
+          WarnS("cannot set option");
 #if 0
         if (TEST_OPT_INTSTRATEGY && (currRing!=NULL)
         && rField_has_simple_inverse()
@@ -664,7 +669,7 @@ BOOLEAN setOption(leftv res, leftv v)
           si_opt_1 &= optionStruct[i].resetval;
         }
         else
-          Warn("cannot clear option");
+          WarnS("cannot clear option");
         goto okay;
       }
     }
@@ -676,7 +681,7 @@ BOOLEAN setOption(leftv res, leftv v)
         #ifdef YYDEBUG
         #if YYDEBUG
         /*debugging the bison grammar --> grammar.cc*/
-        extern int    yydebug;
+        EXTERN_VAR int    yydebug;
         if (BVERBOSE(V_YACC)) yydebug=1;
         else                  yydebug=0;
         #endif
@@ -690,7 +695,7 @@ BOOLEAN setOption(leftv res, leftv v)
         #ifdef YYDEBUG
         #if YYDEBUG
         /*debugging the bison grammar --> grammar.cc*/
-        extern int    yydebug;
+        EXTERN_VAR int    yydebug;
         if (BVERBOSE(V_YACC)) yydebug=1;
         else                  yydebug=0;
         #endif
@@ -707,7 +712,6 @@ BOOLEAN setOption(leftv res, leftv v)
   } while (v!=NULL);
 
    // set global variable to show memory usage
-  extern int om_sing_opt_show_mem;
   if (BVERBOSE(V_SHOW_MEM)) om_sing_opt_show_mem = 1;
   else om_sing_opt_show_mem = 0;
 
@@ -778,13 +782,13 @@ const char *singular_date=__DATE__ " " __TIME__;
 char * versionString(/*const bool bShowDetails = false*/ )
 {
   StringSetS("");
-  StringAppend("Singular for %s version %s (%d, %d bit) %s #%s",
+  StringAppend("Singular for %s version %s (%d, %d bit) %s",
                S_UNAME, VERSION, // SINGULAR_VERSION,
                SINGULAR_VERSION, sizeof(void*)*8,
 #ifdef MAKE_DISTRIBUTION
-	       VERSION_DATE, GIT_VERSION);
+               VERSION_DATE);
 #else
-	       singular_date, GIT_VERSION);
+               singular_date);
 #endif
   StringAppendS("\nwith\n\t");
 
@@ -796,21 +800,26 @@ char * versionString(/*const bool bShowDetails = false*/ )
               StringAppend("GMP(%s),", gmp_version);
 #endif
 #ifdef HAVE_NTL
-#include <NTL/version.h>
               StringAppend("NTL(%s),",NTL_VERSION);
 #endif
 
 #ifdef HAVE_FLINT
-              StringAppend("FLINT(%s),",version);
+              StringAppend("FLINT(%s),",FLINT_VERSION);
+#endif
+//              StringAppendS("factory(" FACTORYVERSION "),\n\t");
+              StringAppendS("\n\t");
+#ifndef HAVE_OMALLOC
+              StringAppendS("xalloc,");
+#else
+              StringAppendS("omalloc,");
 #endif
-              StringAppend("factory(%s),\n\t", factoryVersion);
 #if defined(HAVE_DYN_RL)
               if (fe_fgets_stdin==fe_fgets_dummy)
                 StringAppendS("no input,");
               else if (fe_fgets_stdin==fe_fgets)
                 StringAppendS("fgets,");
               if (fe_fgets_stdin==fe_fgets_stdin_drl)
-                StringAppendS("dynamic readline,");
+                StringAppend("dynamic readline%d),",RL_VERSION_MAJOR);
               #ifdef HAVE_FEREAD
               else if (fe_fgets_stdin==fe_fgets_stdin_emu)
                 StringAppendS("emulated readline,");
@@ -819,7 +828,7 @@ char * versionString(/*const bool bShowDetails = false*/ )
                 StringAppendS("unknown fgets method,");
 #else
   #if defined(HAVE_READLINE) && !defined(FEREAD)
-              StringAppendS("static readline,");
+              StringAppend("static readline(%d),",RL_VERSION_MAJOR);
   #else
     #ifdef HAVE_FEREAD
               StringAppendS("emulated readline,");
@@ -831,6 +840,9 @@ char * versionString(/*const bool bShowDetails = false*/ )
 #ifdef HAVE_PLURAL
               StringAppendS("Plural,");
 #endif
+#ifdef HAVE_VSPACE
+              StringAppendS("vspace,");
+#endif
 #ifdef HAVE_DBM
               StringAppendS("DBM,\n\t");
 #else
@@ -843,9 +855,6 @@ char * versionString(/*const bool bShowDetails = false*/ )
 #if YYDEBUG
               StringAppendS("YYDEBUG=1,");
 #endif
-#ifdef HAVE_ASSUME
-             StringAppendS("ASSUME,");
-#endif
 #ifdef MDEBUG
               StringAppend("MDEBUG=%d,",MDEBUG);
 #endif
@@ -867,6 +876,7 @@ char * versionString(/*const bool bShowDetails = false*/ )
 #ifdef KDEBUG
               StringAppendS("KDEBUG,");
 #endif
+              StringAppendS("\n\t");
 #ifdef __OPTIMIZE__
               StringAppendS("CC:OPTIMIZE,");
 #endif
@@ -876,6 +886,22 @@ char * versionString(/*const bool bShowDetails = false*/ )
 #ifdef __NO_INLINE__
               StringAppendS("CC:NO_INLINE,");
 #endif
+#ifdef HAVE_GENERIC_ADD
+              StringAppendS("GenericAdd,");
+#else
+              StringAppendS("AvoidBranching,");
+#endif
+#ifdef HAVE_GENERIC_MULT
+              StringAppendS("GenericMult,");
+#else
+              StringAppendS("TableMult,");
+#endif
+#ifdef HAVE_INVTABLE
+              StringAppendS("invTable,");
+#else
+              StringAppendS("no invTable,");
+#endif
+              StringAppendS("\n\t");
 #ifdef HAVE_EIGENVAL
               StringAppendS("eigenvalues,");
 #endif
@@ -901,7 +927,9 @@ char * versionString(/*const bool bShowDetails = false*/ )
 #ifdef __GNUC__
               "(ver: " __VERSION__ ")"
 #endif
-              "\n",AC_CONFIGURE_ARGS, CC,CFLAGS, CXX,CXXFLAGS,  DEFS,CPPFLAGS,  LDFLAGS,LIBS);
+              "\n",AC_CONFIGURE_ARGS, CC,CFLAGS " " PTHREAD_CFLAGS,
+              CXX,CXXFLAGS " " PTHREAD_CFLAGS,  DEFS,CPPFLAGS,  LDFLAGS,
+              LIBS " " PTHREAD_LIBS);
               feStringAppendResources(0);
               feStringAppendBrowsers(0);
               StringAppendS("\n");
@@ -1029,7 +1057,9 @@ void checkall()
       {
         if (IDTYP(hh)==PACKAGE_CMD)
         {
-          idhdl h2=IDPACKAGE(hh)->idroot;
+          idhdl h2=NULL;
+          if (IDPACKAGE(hh)!=NULL)
+            h2=IDPACKAGE(hh)->idroot;
           if (IDPACKAGE(hh)!=basePack)
           {
             while (h2!=NULL)
@@ -1049,10 +1079,6 @@ void checkall()
 }
 #endif
 
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <unistd.h>
-
 extern "C"
 int singular_fstat(int fd, struct stat *buf)
 {
@@ -1073,7 +1099,7 @@ void m2_end(int i)
 {
   if (!m2_end_called)
   {
-    extern FILE* File_Profiling;
+    EXTERN_VAR FILE* File_Profiling;
     if (File_Profiling!=NULL) { fclose(File_Profiling); File_Profiling=NULL; }
     m2_end_called = TRUE;
 #ifdef HAVE_SIMPLEIPC
@@ -1137,6 +1163,9 @@ void m2_end(int i)
     {
       if (i<=0)
       {
+        //extern long all_farey;
+        //extern long farey_cnt;
+        //if (all_farey!=0L) printf("farey:%ld, cnt=%ld\n",all_farey,farey_cnt);
         if (TEST_V_QUIET)
         {
           if (i==0)
@@ -1174,35 +1203,14 @@ extern "C"
   }
 }
 
-#ifdef SINGULAR_4_2
-static n_coeffType n_pAE=n_unknown;
-static BOOLEAN ii_pAE_init(leftv res,leftv a)
-{
-  if (a->Typ()!=INT_CMD)
-  {
-    WerrorS("`int` expected");
-    return TRUE;
-  }
-  else
-  {
-    res->rtyp=CRING_CMD;
-    res->data=(void*)nInitChar(n_pAE,(void*)a->Data());
-    return FALSE;
-  }
-}
-#endif
 #ifdef HAVE_FLINT
-static n_coeffType n_FlintZn=n_unknown;
+STATIC_VAR n_coeffType n_FlintZn=n_unknown;
+STATIC_VAR n_coeffType n_FlintQ=n_unknown;
+//STATIC_VAR n_coeffType n_FlintQrat=n_unknown;
 static BOOLEAN ii_FlintZn_init(leftv res,leftv a)
 {
-  if ((a->Typ()!=INT_CMD)
-  ||(a->next==NULL)
-  ||(a->next->Typ()!=STRING_CMD))
-  {
-    WerrorS("`int`i,`string` expected");
-    return TRUE;
-  }
-  else
+  const short t[]={2,INT_CMD,STRING_CMD};
+  if (iiCheckTypes(a,t,1))
   {
     flintZn_struct p;
     p.ch=(int)(long)a->Data();
@@ -1211,22 +1219,144 @@ static BOOLEAN ii_FlintZn_init(leftv res,leftv a)
     res->data=(void*)nInitChar(n_FlintZn,(void*)&p);
     return FALSE;
   }
+  return TRUE;
+}
+static BOOLEAN ii_FlintQ_init(leftv res,leftv a)
+{
+  const short t[]={1,STRING_CMD};
+  if (iiCheckTypes(a,t,1))
+  {
+    char* p;
+    p=(char*)a->Data();
+    res->rtyp=CRING_CMD;
+    res->data=(void*)nInitChar(n_FlintQ,(void*)p);
+    return FALSE;
+  }
+  return TRUE;
+}
+#if __FLINT_RELEASE >= 20503
+static BOOLEAN ii_FlintQrat_init(leftv res,leftv a)
+{
+  if (a==NULL)
+  {
+    WerrorS("at least one name required");
+    return TRUE;
+  }
+  QaInfo par;
+  #ifdef QA_DEBUG
+  par.C=r->cf;
+  a=a->next;
+  #endif
+  par.N=a->listLength();
+  par.names=(char**)omAlloc(par.N*sizeof(char*));
+  int i=0;
+  while(a!=NULL)
+  {
+    par.names[i]=omStrDup(a->Name());
+    i++;
+    a=a->next;
+  }
+  res->rtyp=CRING_CMD;
+  res->data=(void*)nInitChar(n_FlintQrat,&par);
+  for(i=par.N-1;i>=0;i--)
+  {
+    omFree(par.names[i]);
+  }
+  omFreeSize(par.names,par.N*sizeof(char*));
+  return FALSE;
 }
 #endif
+extern "C" int flint_mod_init(SModulFunctions* psModulFunctions)
+{
+    package save=currPack;
+    currPack=basePack;
+    n_FlintQ=nRegister(n_unknown,flintQ_InitChar);
+    if (n_FlintQ!=n_unknown)
+    {
+      iiAddCproc("kernel","flintQp",FALSE,ii_FlintQ_init);
+      nRegisterCfByName(flintQInitCfByName,n_FlintQ);
+    }
+#if __FLINT_RELEASE >= 20503
+    iiAddCproc("kernel","flintQ",FALSE,ii_FlintQrat_init);
+    nRegisterCfByName(flintQInitCfByName,n_FlintQ);
+#endif
+    n_FlintZn=nRegister(n_unknown,flintZn_InitChar);
+    if (n_FlintZn!=n_unknown)
+    {
+      iiAddCproc("kernel","flintZn",FALSE,ii_FlintZn_init);
+      nRegisterCfByName(flintZnInitCfByName,n_FlintZn);
+    }
+    currPack=save;
+    return MAX_TOK;
+}
+#endif
+
+static BOOLEAN iiFloat(leftv res, leftv pnn)
+{
+  short float_len=3;
+  short float_len2=SHORT_REAL_LENGTH;
+  coeffs cf=NULL;
+  if ((pnn!=NULL) && (pnn->Typ()==INT_CMD))
+  {
+    float_len=(int)(long)pnn->Data();
+    float_len2=float_len;
+    pnn=pnn->next;
+    if ((pnn!=NULL) && (pnn->Typ()==INT_CMD))
+    {
+      float_len2=(int)(long)pnn->Data();
+      pnn=pnn->next;
+    }
+  }
+  if (float_len2 <= (short)SHORT_REAL_LENGTH)
+       cf=nInitChar(n_R, NULL);
+  else // longR or longC?
+  {
+    LongComplexInfo param;
+    param.float_len = si_min (float_len, 32767);
+    param.float_len2 = si_min (float_len2, 32767);
+    cf = nInitChar(n_long_R, (void*)&param);
+  }
+  res->rtyp=CRING_CMD;
+  res->data=cf;
+  return cf==NULL;
+}
+static BOOLEAN iiCrossProd(leftv res, leftv args)
+{
+  leftv h=args;
+  coeffs *c=NULL;
+  coeffs cf=NULL;
+  int i=0;
+  if (h==NULL) goto crossprod_error;
+  while (h!=NULL)
+  {
+    if (h->Typ()!=CRING_CMD) goto crossprod_error;
+    i++;
+    h=h->next;
+  }
+  c=(coeffs*)omAlloc0((i+1)*sizeof(coeffs));
+  h=args;
+  i=0;
+  while (h!=NULL)
+  {
+    c[i]=(coeffs)h->CopyD();
+    i++;
+    h=h->next;
+  }
+  cf=nInitChar(n_nTupel,c);
+  res->data=cf;
+  res->rtyp=CRING_CMD;
+  return FALSE;
+
+  crossprod_error:
+    WerrorS("expected `crossprod(coeffs, ...)`");
+    return TRUE;
+}
 /*2
 * initialize components of Singular
 */
+static void callWerrorS(const char *s) { WerrorS(s); }
 void siInit(char *name)
 {
-// factory default settings: -----------------------------------------------
-  On(SW_USE_EZGCD);
-  On(SW_USE_CHINREM_GCD);
-  //On(SW_USE_FF_MOD_GCD);
-  On(SW_USE_EZGCD_P);
-  On(SW_USE_QGCD);
-  Off(SW_USE_NTL_SORT); // may be changed by an command line option
-  factoryError=WerrorS;
-
 // memory initialization: -----------------------------------------------
     om_Opts.OutOfMemoryFunc = omSingOutOfMemoryFunc;
 #ifndef OM_NDEBUG
@@ -1239,7 +1369,11 @@ void siInit(char *name)
     om_Opts.Keep = 0; /* OM_NDEBUG */
 #endif
     omInitInfo();
-
+// factory
+#ifndef HAVE_NTL
+  extern void initPT();
+  initPT();
+#endif
 // options ---------------------------------------------------------------
   si_opt_1=0;
 // interpreter tables etc.: -----------------------------------------------
@@ -1251,9 +1385,9 @@ void siInit(char *name)
   basePack=(package)omAlloc0(sizeof(*basePack));
   currPack=basePack;
   idhdl h;
-  h=enterid("Top", 0, PACKAGE_CMD, &IDROOT, TRUE);
-  IDPACKAGE(h)->language = LANG_TOP;
+  h=enterid("Top", 0, PACKAGE_CMD, &IDROOT, FALSE);
   IDPACKAGE(h)=basePack;
+  IDPACKAGE(h)->language = LANG_TOP;
   currPackHdl=h;
   basePackHdl=h;
 
@@ -1296,7 +1430,7 @@ void siInit(char *name)
 // singular links: --------------------------------------------------
   slStandardInit();
   myynest=0;
-// semapohore 0 -----------------------------------------------------
+// how many processes ? -----------------------------------------------------
   int cpus=2;
   int cpu_n;
   #ifdef _SC_NPROCESSORS_ONLN
@@ -1305,51 +1439,23 @@ void siInit(char *name)
   if ((cpu_n=sysconf(_SC_NPROCESSORS_CONF))>cpus) cpus=cpu_n;
   #endif
   feSetOptValue(FE_OPT_CPUS, cpus);
+// how many threads ? -----------------------------------------------------
+  feSetOptValue(FE_OPT_THREADS, cpus);
 
 // default coeffs
   {
     idhdl h;
-    h=enterid(omStrDup("QQ"),0/*level*/, CRING_CMD,&(basePack->idroot),FALSE /*init*/,FALSE /*search*/);
+    h=enterid("QQ",0/*level*/, CRING_CMD,&(basePack->idroot),FALSE /*init*/,FALSE /*search*/);
     IDDATA(h)=(char*)nInitChar(n_Q,NULL);
-    h=enterid(omStrDup("ZZ"),0/*level*/, CRING_CMD,&(basePack->idroot),FALSE /*init*/,FALSE /*search*/);
+    h=enterid("ZZ",0/*level*/, CRING_CMD,&(basePack->idroot),FALSE /*init*/,FALSE /*search*/);
     IDDATA(h)=(char*)nInitChar(n_Z,NULL);
-    //h=enterid(omStrDup("RR"),0/*level*/, CRING_CMD,&(basePack->idroot),FALSE /*init*/,FALSE /*search*/);
+    nRegisterCfByName(nrnInitCfByName,n_Zn); // and n_Znm
+    iiAddCproc("kernel","crossprod",FALSE,iiCrossProd);
+    iiAddCproc("kernel","Float",FALSE,iiFloat);
+    //h=enterid("RR",0/*level*/, CRING_CMD,&(basePack->idroot),FALSE /*init*/,FALSE /*search*/);
     //IDDATA(h)=(char*)nInitChar(n_R,NULL);
-    //h=enterid(omStrDup("CC"),0/*level*/, CRING_CMD,&(basePack->idroot),FALSE /*init*/,FALSE /*search*/);
+    //h=enterid("CC",0/*level*/, CRING_CMD,&(basePack->idroot),FALSE /*init*/,FALSE /*search*/);
     //IDDATA(h)=(char*)nInitChar(n_long_C,NULL);
-    n_coeffType t;
-#ifdef SINGULAR_4_2
-    t=nRegister(n_unknown,n_AEInitChar);
-    if (t!=n_unknown)
-    {
-      h=enterid(omStrDup("AE"),0/*level*/, CRING_CMD,&(basePack->idroot),FALSE /*init*/,FALSE /*search*/);
-      IDDATA(h)=(char*)nInitChar(t,NULL);
-    }
-    t=nRegister(n_unknown,n_QAEInitChar);
-    if (t!=n_unknown)
-    {
-      h=enterid(omStrDup("QAE"),0/*level*/, CRING_CMD,&(basePack->idroot),FALSE /*init*/,FALSE /*search*/);
-      IDDATA(h)=(char*)nInitChar(t,NULL);
-    }
-    n_pAE=nRegister(n_unknown,n_pAEInitChar);
-    if (n_pAE!=n_unknown)
-    {
-      iiAddCproc("kernel","pAE",FALSE,ii_pAE_init);
-    }
-#endif
-    #ifdef HAVE_FLINT
-    t=nRegister(n_unknown,flintQ_InitChar);
-    if (t!=n_unknown)
-    {
-      h=enterid(omStrDup("flint_poly_Q"),0/*level*/, CRING_CMD,&(basePack->idroot),FALSE /*init*/,FALSE /*search*/);
-      IDDATA(h)=(char*)nInitChar(t,NULL);
-    }
-    n_FlintZn=nRegister(n_unknown,flintZn_InitChar);
-    if (n_FlintZn!=n_unknown)
-    {
-      iiAddCproc("kernel","flintZ",FALSE,ii_FlintZn_init);
-    }
-    #endif
   }
 // setting routines for PLURAL QRINGS:
 // allowing to use libpolys without libSingular(kStd)
@@ -1367,8 +1473,10 @@ void siInit(char *name)
     BITSET save1,save2;
     SI_SAVE_OPT(save1,save2);
     si_opt_2 &= ~Sy_bit(V_LOAD_LIB);
-    iiLibCmd(omStrDup("standard.lib"), TRUE,TRUE,TRUE);
+    iiLibCmd("standard.lib", TRUE,TRUE,TRUE);
     SI_RESTORE_OPT(save1,save2);
   }
+  // interpreter error handling
+  factoryError=callWerrorS; // to honour later changes of variable WerrorS
   errorreported = 0;
 }
