@@ -4,63 +4,71 @@
 /*
 * ABSTRACT: numbers modulo 2^m
 */
-#include <misc/auxiliary.h>
+#include "misc/auxiliary.h"
 
-#include <omalloc/omalloc.h>
+#include "misc/mylimits.h"
+#include "reporter/reporter.h"
 
-#include <misc/mylimits.h>
-#include <reporter/reporter.h>
+#include "coeffs/si_gmp.h"
+#include "coeffs/coeffs.h"
+#include "coeffs/numbers.h"
+#include "coeffs/longrat.h"
+#include "coeffs/mpr_complex.h"
 
-#include "si_gmp.h"
-#include "coeffs.h"
-#include "numbers.h"
-#include "longrat.h"
-#include "mpr_complex.h"
-
-#include "rmodulo2m.h"
-#include "rmodulon.h"
+#include "coeffs/rmodulo2m.h"
+#include "coeffs/rmodulon.h"
 
 #include <string.h>
 
 #ifdef HAVE_RINGS
 
+#ifdef LDEBUG
+BOOLEAN nr2mDBTest(number a, const char *f, const int l, const coeffs r)
+{
+  if (((long)a<0L) || ((long)a>(long)r->mod2mMask))
+  {
+    Print("wrong mod 2^n number %ld at %s,%d\n",(long)a,f,l);
+    return FALSE;
+  }
+  return TRUE;
+}
+#endif
+
+
 static inline number nr2mMultM(number a, number b, const coeffs r)
 {
   return (number)
-    ((((unsigned long) a) * ((unsigned long) b)) & ((unsigned long)r->mod2mMask));
+    ((((unsigned long) a) * ((unsigned long) b)) & r->mod2mMask);
 }
 
 static inline number nr2mAddM(number a, number b, const coeffs r)
 {
   return (number)
-    ((((unsigned long) a) + ((unsigned long) b)) & ((unsigned long)r->mod2mMask));
+    ((((unsigned long) a) + ((unsigned long) b)) & r->mod2mMask);
 }
 
 static inline number nr2mSubM(number a, number b, const coeffs r)
 {
   return (number)((unsigned long)a < (unsigned long)b ?
-                       r->mod2mMask - (unsigned long)b + (unsigned long)a + 1:
+                       r->mod2mMask+1 - (unsigned long)b + (unsigned long)a:
                        (unsigned long)a - (unsigned long)b);
 }
 
-#define nr2mNegM(A,r) (number)((r->mod2mMask - (unsigned long)(A) + 1) & r->mod2mMask)
+#define nr2mNegM(A,r) (number)((r->mod2mMask+1 - (unsigned long)(A)) & r->mod2mMask)
 #define nr2mEqualM(A,B)  ((A)==(B))
 
-extern omBin gmp_nrz_bin; /* init in rintegers*/
+EXTERN_VAR omBin gmp_nrz_bin; /* init in rintegers*/
 
 static char* nr2mCoeffName(const coeffs cf)
 {
-  static char n2mCoeffName_buf[22];
-  snprintf(n2mCoeffName_buf,21,"ZZ/(2^%lu)",cf->modExponent);
+  STATIC_VAR char n2mCoeffName_buf[30];
+  if (cf->modExponent>32) /* for 32/64bit arch.*/
+    snprintf(n2mCoeffName_buf,21,"ZZ/(bigint(2)^%lu)",cf->modExponent);
+  else
+    snprintf(n2mCoeffName_buf,21,"ZZ/(2^%lu)",cf->modExponent);
   return n2mCoeffName_buf;
 }
 
-static void    nr2mCoeffWrite  (const coeffs r, BOOLEAN /*details*/)
-{
-  PrintS("//   coeff. ring is : ");
-  Print("Z/2^%lu\n", r->modExponent);
-}
-
 static BOOLEAN nr2mCoeffIsEqual(const coeffs r, n_coeffType n, void * p)
 {
   if (n==n_Z2m)
@@ -72,50 +80,42 @@ static BOOLEAN nr2mCoeffIsEqual(const coeffs r, n_coeffType n, void * p)
   return FALSE;
 }
 
-static char* nr2mCoeffString(const coeffs r)
-{
-  // r->modExponent <=bitsize(long)
-  char* s = (char*) omAlloc(11+11);
-  sprintf(s,"ZZ/(2^%lu)",r->modExponent);
-  return s;
-}
-
 static coeffs nr2mQuot1(number c, const coeffs r)
 {
-    coeffs rr;
-    long ch = r->cfInt(c, r);
-    mpz_t a,b;
-    mpz_init_set(a, r->modNumber);
-    mpz_init_set_ui(b, ch);
-    mpz_ptr gcd;
-    gcd = (mpz_ptr) omAlloc(sizeof(mpz_t));
-    mpz_init(gcd);
-    mpz_gcd(gcd, a,b);
-    if(mpz_cmp_ui(gcd, 1) == 0)
-        {
-            WerrorS("constant in q-ideal is coprime to modulus in ground ring");
-            WerrorS("Unable to create qring!");
-            return NULL;
-        }
-    if(mpz_cmp_ui(gcd, 2) == 0)
+  coeffs rr;
+  long ch = r->cfInt(c, r);
+  mpz_t a,b;
+  mpz_init_set(a, r->modNumber);
+  mpz_init_set_ui(b, ch);
+  mpz_ptr gcd;
+  gcd = (mpz_ptr) omAlloc(sizeof(mpz_t));
+  mpz_init(gcd);
+  mpz_gcd(gcd, a,b);
+  if(mpz_cmp_ui(gcd, 1) == 0)
+  {
+    WerrorS("constant in q-ideal is coprime to modulus in ground ring");
+    WerrorS("Unable to create qring!");
+    return NULL;
+  }
+  if(mpz_cmp_ui(gcd, 2) == 0)
+  {
+    rr = nInitChar(n_Zp, (void*)2);
+  }
+  else
+  {
+    int kNew = 1;
+    mpz_t baseTokNew;
+    mpz_init(baseTokNew);
+    mpz_set(baseTokNew, r->modBase);
+    while(mpz_cmp(gcd, baseTokNew) > 0)
     {
-        rr = nInitChar(n_Zp, (void*)2);
+      kNew++;
+      mpz_mul(baseTokNew, baseTokNew, r->modBase);
     }
-    else
-    {
-        int kNew = 1;
-        mpz_t baseTokNew;
-        mpz_init(baseTokNew);
-        mpz_set(baseTokNew, r->modBase);
-        while(mpz_cmp(gcd, baseTokNew) > 0)
-        {
-          kNew++;
-          mpz_mul(baseTokNew, baseTokNew, r->modBase);
-        }
-        mpz_clear(baseTokNew);
-        rr = nInitChar(n_Z2m, (void*)(long)kNew);
-    }
-    return(rr);
+    mpz_clear(baseTokNew);
+    rr = nInitChar(n_Z2m, (void*)(long)kNew);
+  }
+  return(rr);
 }
 
 /* TRUE iff 0 < k <= 2^m / 2 */
@@ -131,10 +131,13 @@ static BOOLEAN nr2mGreaterZero(number k, const coeffs r)
  */
 static number nr2mMult(number a, number b, const coeffs r)
 {
+  number n;
   if (((unsigned long)a == 0) || ((unsigned long)b == 0))
     return (number)0;
   else
-    return nr2mMultM(a, b, r);
+    n=nr2mMultM(a, b, r);
+  n_Test(n,r);
+  return n;
 }
 
 static number nr2mAnn(number b, const coeffs r);
@@ -212,7 +215,7 @@ static void specialXGCD(unsigned long& s, unsigned long a, const coeffs r)
   mpz_ptr rr = (mpz_ptr)omAlloc(sizeof(mpz_t));
   mpz_init(rr);
 
-  while (mpz_cmp_ui(v, 0) != 0) /* i.e., while v != 0 */
+  while (mpz_sgn1(v) != 0) /* i.e., while v != 0 */
   {
     mpz_div(q, u, v);
     mpz_mod(rr, u, v);
@@ -226,7 +229,7 @@ static void specialXGCD(unsigned long& s, unsigned long a, const coeffs r)
     mpz_set(v1, v0);
   }
 
-  while (mpz_cmp_ui(u1, 0) < 0) /* i.e., while u1 < 0 */
+  while (mpz_sgn1(u1) < 0) /* i.e., while u1 < 0 */
   {
     /* we add 2^m = (2^m - 1) + 1 to u1: */
     mpz_add_ui(u1, u1, r->mod2mMask);
@@ -340,7 +343,7 @@ static number nr2mInit(long i, const coeffs r)
  */
 static long nr2mInt(number &n, const coeffs r)
 {
-  unsigned long nn = (unsigned long)(unsigned long)n & r->mod2mMask;
+  unsigned long nn = (unsigned long)n;
   unsigned long l = r->mod2mMask >> 1; l++; /* now: l = 2^(m-1) */
   if ((unsigned long)nn > l)
     return (long)((unsigned long)nn - r->mod2mMask - 1);
@@ -350,12 +353,16 @@ static long nr2mInt(number &n, const coeffs r)
 
 static number nr2mAdd(number a, number b, const coeffs r)
 {
-  return nr2mAddM(a, b, r);
+  number n=nr2mAddM(a, b, r);
+  n_Test(n,r);
+  return n;
 }
 
 static number nr2mSub(number a, number b, const coeffs r)
 {
-  return nr2mSubM(a, b, r);
+  number n=nr2mSubM(a, b, r);
+  n_Test(n,r);
+  return n;
 }
 
 static BOOLEAN nr2mIsUnit(number a, const coeffs)
@@ -404,14 +411,21 @@ static number nr2mDiv(number a, number b, const coeffs r)
         b = (number)((unsigned long)b / 2);
       }
     }
-    if ((unsigned long)b % 2 == 0)
+    if ((long)b==0L)
+    {
+      WerrorS(nDivBy0);
+      return (number)0L;
+    }
+    else if ((unsigned long)b % 2 == 0)
     {
       WerrorS("Division not possible, even by cancelling zero divisors.");
       WerrorS("Result is integer division without remainder.");
       return (number) ((unsigned long) a / (unsigned long) b);
     }
   }
-  return (number)nr2mMult(a, nr2mInversM(b,r),r);
+  number n=(number)nr2mMult(a, nr2mInversM(b,r),r);
+  n_Test(n,r);
+  return n;
 }
 
 /* Is 'a' divisible by 'b'? There are two cases:
@@ -578,12 +592,14 @@ static number nr2mAnn(number b, const coeffs r)
 static number nr2mNeg(number c, const coeffs r)
 {
   if ((unsigned long)c == 0) return c;
-  return nr2mNegM(c, r);
+  number n=nr2mNegM(c, r);
+  n_Test(n,r);
+  return n;
 }
 
 static number nr2mMapMachineInt(number from, const coeffs /*src*/, const coeffs dst)
 {
-  unsigned long i = ((unsigned long)from) % dst->mod2mMask ;
+  unsigned long i = ((unsigned long)from) % (dst->mod2mMask + 1) ;
   return (number)i;
 }
 
@@ -623,8 +639,7 @@ static number nr2mMapGMP(number from, const coeffs /*src*/, const coeffs dst)
 static number nr2mMapQ(number from, const coeffs src, const coeffs dst)
 {
   mpz_ptr gmp = (mpz_ptr)omAllocBin(gmp_nrz_bin);
-  mpz_init(gmp);
-  nlGMP(from, (number)gmp, src); // FIXME? TODO? // extern void   nlGMP(number &i, number n, const coeffs r); // to be replaced with n_MPZ(erg, from, src); // ?
+  nlMPZ(gmp, from, src);
   number res=nr2mMapGMP((number)gmp,src,dst);
   mpz_clear(gmp); omFree((ADDRESS)gmp);
   return res;
@@ -642,11 +657,6 @@ static number nr2mMapZ(number from, const coeffs src, const coeffs dst)
 
 static nMapFunc nr2mSetMap(const coeffs src, const coeffs dst)
 {
-  if ((src->rep==n_rep_int) && nCoeff_is_Ring_2toM(src)
-     && (src->mod2mMask == dst->mod2mMask))
-  {
-    return ndCopyMap;
-  }
   if ((src->rep==n_rep_int) && nCoeff_is_Ring_2toM(src)
      && (src->mod2mMask < dst->mod2mMask))
   { /* i.e. map an integer mod 2^s into Z mod 2^t, where t < s */
@@ -658,15 +668,15 @@ static nMapFunc nr2mSetMap(const coeffs src, const coeffs dst)
     // to be done
     return nr2mMapProject;
   }
-  if ((src->rep==n_rep_gmp) && nCoeff_is_Ring_Z(src))
+  if ((src->rep==n_rep_gmp) && nCoeff_is_Z(src))
   {
     return nr2mMapGMP;
   }
-  if ((src->rep==n_rep_gap_gmp) /*&& nCoeff_is_Ring_Z(src)*/)
+  if ((src->rep==n_rep_gap_gmp) /*&& nCoeff_is_Z(src)*/)
   {
     return nr2mMapZ;
   }
-  if ((src->rep==n_rep_gap_rat) && (nCoeff_is_Q(src)||nCoeff_is_Ring_Z(src)))
+  if ((src->rep==n_rep_gap_rat) && (nCoeff_is_Q(src)||nCoeff_is_Z(src)))
   {
     return nr2mMapQ;
   }
@@ -675,7 +685,7 @@ static nMapFunc nr2mSetMap(const coeffs src, const coeffs dst)
     return nr2mMapZp;
   }
   if ((src->rep==n_rep_gmp) &&
-  (nCoeff_is_Ring_PtoM(src) || nCoeff_is_Ring_ModN(src)))
+  (nCoeff_is_Ring_PtoM(src) || nCoeff_is_Zn(src)))
   {
     if (mpz_divisible_2exp_p(src->modNumber,dst->modExponent))
       return nr2mMapGMP;
@@ -712,15 +722,6 @@ static void nr2mInitExp(int m, coeffs r)
     WarnS("nr2mInitExp unexpectedly called with m = 1 (we continue with Z/2^2");
 }
 
-#ifdef LDEBUG
-static BOOLEAN nr2mDBTest (number a, const char *, const int, const coeffs r)
-{
-  //if ((unsigned long)a < 0) return FALSE; // is unsigned!
-  if (((unsigned long)a & r->mod2mMask) != (unsigned long)a) return FALSE;
-  return TRUE;
-}
-#endif
-
 static void nr2mWrite (number a, const coeffs r)
 {
   long i = nr2mInt(a, r);
@@ -776,7 +777,6 @@ BOOLEAN nr2mInitChar (coeffs r, void* p)
 
   //r->cfKillChar    = ndKillChar; /* dummy*/
   r->nCoeffIsEqual = nr2mCoeffIsEqual;
-  r->cfCoeffString = nr2mCoeffString;
 
   r->modBase = (mpz_ptr) omAllocBin (gmp_nrz_bin);
   mpz_init_set_si (r->modBase, 2L);
@@ -817,7 +817,6 @@ BOOLEAN nr2mInitChar (coeffs r, void* p)
   r->cfIsUnit      = nr2mIsUnit;
   r->cfGetUnit     = nr2mGetUnit;
   r->cfExtGcd      = nr2mExtGcd;
-  r->cfCoeffWrite  = nr2mCoeffWrite;
   r->cfCoeffName   = nr2mCoeffName;
   r->cfQuot1       = nr2mQuot1;
 #ifdef LDEBUG
