@@ -8,26 +8,20 @@
 
 //#define FACTORIZE2_DEBUG
 
-
-
-
-
-
-
-#include <misc/auxiliary.h>
+#include "misc/auxiliary.h"
 #include "clapsing.h"
 
-#include <factory/factory.h>
+#include "factory/factory.h"
+#include "factory/cf_roots.h"
 
-#include <omalloc/omalloc.h>
-#include <coeffs/numbers.h>
-#include <coeffs/coeffs.h>
-#include <coeffs/bigintmat.h>
-
-// #include <kernel/ffields.h>
+#include "coeffs/numbers.h"
+#include "coeffs/coeffs.h"
+#include "coeffs/bigintmat.h"
 
 #include "monomials/ring.h"
 #include "simpleideals.h"
+#include "polys/flintconv.h"
+#include "polys/flint_mpoly.h"
 
 
 //#include "polys.h"
@@ -37,14 +31,13 @@
 
 
 #include "clapconv.h"
-// #include <kernel/clapconv.h>
 
-#include <polys/monomials/p_polys.h>
-#include <polys/monomials/ring.h>
-#include <polys/simpleideals.h>
-#include <misc/intvec.h>
-#include <polys/matpol.h>
-#include <coeffs/bigintmat.h>
+#include "polys/monomials/p_polys.h"
+#include "polys/monomials/ring.h"
+#include "polys/simpleideals.h"
+#include "misc/intvec.h"
+#include "polys/matpol.h"
+#include "coeffs/bigintmat.h"
 
 
 void out_cf(const char *s1,const CanonicalForm &f,const char *s2);
@@ -56,66 +49,50 @@ poly singclap_gcd_r ( poly f, poly g, const ring r )
   assume(f!=NULL);
   assume(g!=NULL);
 
-  if(pNext(f)==NULL && pNext(g)==NULL)
+  if(pNext(f)==NULL)
   {
-    poly p=p_One(r);
-    for(int i=rVar(r);i>0;i--)
-      p_SetExp(p,i,si_min(p_GetExp(f,i,r),p_GetExp(g,i,r)),r);
-    if (rField_is_Ring(r))
-    {
-      number c = p_GetCoeff(f,r);
-      number d = p_GetCoeff(g,r);
-      p_SetCoeff(p,n_Gcd(c,d,r->cf),r);
-    }
-    p_Setm(p,r);
-    return p;
-#if 0
-    else
-    {
-      poly h=g;
-      for(int i=rVar(r);i>0;i--)
-        p_SetExp(p,i,p_GetExp(f,i,r),r);
-      while(h!=NULL)
-      {
-        for(int i=rVar(r);i>0;i--)
-          p_SetExp(p,i,si_min(p_GetExp(p,i,r),p_GetExp(h,i,r)),r);
-        pIter(h);
-      }
-      p_Setm(p,r);
-      return p;
-    }
-#endif
+    return p_GcdMon(f,g,r);
   }
-#if 0
-  else if (pNext(g)==NULL)
+  else if(pNext(g)==NULL)
   {
-    poly p=p_One(r);
-    poly h=f;
-    for(int i=rVar(r);i>0;i--)
-      p_SetExp(p,i,p_GetExp(g,i,r),r);
-    if (rField_is_Ring(r))
-    {
-      number c = p_GetCoeff(f,r);
-      number d = p_GetCoeff(g,r);
-      p_SetCoeff(p,n_Gcd(c,d,r->cf),r);
-    }
-    while(h!=NULL)
-    {
-      for(int i=rVar(r);i>0;i--)
-        p_SetExp(p,i,si_min(p_GetExp(p,i,r),p_GetExp(h,i,r)),r);
-      pIter(h);
-    }
-    p_Setm(p,r);
-    return p;
+    return p_GcdMon(g,f,r);
   }
-#endif
-
+  #ifdef HAVE_FLINT
+  #if __FLINT_RELEASE >= 20503
+  if (rField_is_Zp(r) && (r->cf->ch>10))
+  {
+    nmod_mpoly_ctx_t ctx;
+    if (!convSingRFlintR(ctx,r))
+    {
+      // leading coef. 1
+      return Flint_GCD_MP(f,pLength(f),g,pLength(g),ctx,r);
+    }
+  }
+  else
+  if (rField_is_Q(r))
+  {
+    fmpq_mpoly_ctx_t ctx;
+    if (!convSingRFlintR(ctx,r))
+    {
+      // leading coef. positive, all coeffs in Z
+      poly res=Flint_GCD_MP(f,pLength(f),g,pLength(g),ctx,r);
+      res=p_Cleardenom(res,r);
+      return res;
+    }
+  }
+  #endif
+  #endif
   Off(SW_RATIONAL);
-  if (rField_is_Q(r) || rField_is_Zp(r) || rField_is_Ring_Z(r))
+  if (rField_is_Q(r) || rField_is_Zp(r) || rField_is_Z(r)
+  || (rField_is_Zn(r)&&(r->cf->convSingNFactoryN!=ndConvSingNFactoryN)))
   {
     setCharacteristic( rChar(r) );
     CanonicalForm F( convSingPFactoryP( f,r ) ), G( convSingPFactoryP( g, r ) );
     res=convFactoryPSingP( gcd( F, G ) , r);
+    if ( rField_is_Zp(r))
+      p_Norm(res,r); // leading coef. 1
+    else if (rField_is_Q(r) && (!n_GreaterZero(pGetCoeff(res),r->cf)))
+      res = p_Neg(res,r); // leading coef. positive, all coeffs in Z
   }
   // and over Q(a) / Fp(a)
   else if ( r->cf->extRing!=NULL )
@@ -134,15 +111,24 @@ poly singclap_gcd_r ( poly f, poly g, const ring r )
       res= convFactoryAPSingAP( gcd( F, G ),r );
       prune (a);
       if (!b1) Off(SW_USE_QGCD);
+      if ( rField_is_Zp_a(r)) p_Norm(res,r); // leading coef. 1
     }
     else
     {
+      convSingTrP(f,r);
+      convSingTrP(g,r);
       CanonicalForm F( convSingTrPFactoryP( f,r ) ), G( convSingTrPFactoryP( g,r ) );
       res= convFactoryPSingTrP( gcd( F, G ),r );
     }
   }
-  else
+  else if (r->cf->convSingNFactoryN==ndConvSingNFactoryN)
     WerrorS( feNotImplemented );
+  else
+  { // handle user type coeffs:
+    setCharacteristic( rChar(r) );
+    CanonicalForm F( convSingPFactoryP( f,r ) ), G( convSingPFactoryP( g, r ) );
+    res=convFactoryPSingP( gcd( F, G ) , r);
+  }
   Off(SW_RATIONAL);
   return res;
 }
@@ -163,10 +149,33 @@ poly singclap_gcd_and_divide ( poly& f, poly& g, const ring r)
     g=p_One (r);
     return res;
   }
+  if (pNext(g)==NULL)
+  {
+    poly G=p_GcdMon(g,f,r);
+    if (!n_IsOne(pGetCoeff(G),r->cf)
+    || (!p_IsConstant(G,r)))
+    {
+      f=p_Div_mm(f,G,r);
+      g=p_Div_mm(g,G,r);
+    }
+    return G;
+  }
+  else if (pNext(f)==NULL)
+  {
+    poly G=p_GcdMon(f,g,r);
+    if (!n_IsOne(pGetCoeff(G),r->cf)
+    || (!p_IsConstant(G,r)))
+    {
+      f=p_Div_mm(f,G,r);
+      g=p_Div_mm(g,G,r);
+    }
+    return G;
+  }
 
   Off(SW_RATIONAL);
   CanonicalForm F,G,GCD;
-  if (rField_is_Q(r) || (rField_is_Zp(r)))
+  if (rField_is_Q(r) || (rField_is_Zp(r))
+  || (rField_is_Zn(r)&&(r->cf->convSingNFactoryN!=ndConvSingNFactoryN)))
   {
     bool b1=isOn(SW_USE_EZGCD_P);
     setCharacteristic( rChar(r) );
@@ -284,21 +293,6 @@ poly singclap_gcd_and_divide ( poly& f, poly& g, const ring r)
   return res;
 }
 
-poly singclap_gcd ( poly f, poly g, const ring r)
-{
-  poly res=NULL;
-
-  if (f!=NULL) p_Cleardenom(f, r);
-  if (g!=NULL) p_Cleardenom(g, r);
-  else         return f; // g==0 => gcd=f (but do a p_Cleardenom)
-  if (f==NULL) return g; // f==0 => gcd=g (but do a p_Cleardenom)
-
-  res=singclap_gcd_r(f,g,r);
-  p_Delete(&f, r);
-  p_Delete(&g, r);
-  return res;
-}
-
 /*2 find the maximal exponent of var(i) in poly p*/
 int pGetExp_Var(poly p, int i, const ring r)
 {
@@ -327,7 +321,8 @@ poly singclap_resultant ( poly f, poly g , poly x, const ring r)
     goto resultant_returns_res;
   // for now there is only the possibility to handle polynomials over
   // Q and Fp ...
-  if (rField_is_Zp(r) || rField_is_Q(r))
+  if (rField_is_Zp(r) || rField_is_Q(r)
+  || (rField_is_Zn(r)&&(r->cf->convSingNFactoryN!=ndConvSingNFactoryN)))
   {
     Variable X(i);
     setCharacteristic( rChar(r) );
@@ -368,7 +363,7 @@ poly singclap_resultant ( poly f, poly g , poly x, const ring r)
         number n=n_Invers(nf,r->cf);
         while(eg>0)
         {
-          res=p_Mult_nn(res,n,r);
+          res=__p_Mult_nn(res,n,r);
           eg--;
         }
         n_Delete(&n,r->cf);
@@ -379,7 +374,7 @@ poly singclap_resultant ( poly f, poly g , poly x, const ring r)
         number n=n_Invers(ng,r->cf);
         while(ef>0)
         {
-          res=p_Mult_nn(res,n,r);
+          res=__p_Mult_nn(res,n,r);
           ef--;
         }
         n_Delete(&n,r->cf);
@@ -464,7 +459,8 @@ BOOLEAN singclap_extgcd ( poly f, poly g, poly &res, poly &pa, poly &pb , const
   // Q and Fp ...
   res=NULL;pa=NULL;pb=NULL;
   On(SW_SYMMETRIC_FF);
-  if ( rField_is_Q(r) || rField_is_Zp(r) )
+  if ( rField_is_Q(r) || rField_is_Zp(r)
+  || (rField_is_Zn(r)&&(r->cf->convSingNFactoryN!=ndConvSingNFactoryN)))
   {
     setCharacteristic( rChar(r) );
     CanonicalForm F( convSingPFactoryP( f,r ) ), G( convSingPFactoryP( g,r) );
@@ -544,23 +540,96 @@ BOOLEAN singclap_extgcd ( poly f, poly g, poly &res, poly &pa, poly &pb , const
   return FALSE;
 }
 
-poly singclap_pdivide ( poly f, poly g, const ring r )
+poly singclap_pmult ( poly f, poly g, const ring r )
 {
   poly res=NULL;
   On(SW_RATIONAL);
-  if (rField_is_Zp(r) || rField_is_Q(r))
+  if (rField_is_Zp(r) || rField_is_Q(r) || rField_is_Z(r)
+  || (rField_is_Zn(r)&&(r->cf->convSingNFactoryN!=ndConvSingNFactoryN)))
+  {
+    if (rField_is_Z(r)) Off(SW_RATIONAL);
+    setCharacteristic( rChar(r) );
+    CanonicalForm F( convSingPFactoryP( f,r ) ), G( convSingPFactoryP( g,r ) );
+    res = convFactoryPSingP( F * G,r );
+  }
+  else if (r->cf->extRing!=NULL)
+  {
+    if (rField_is_Q_a(r)) setCharacteristic( 0 );
+    else               setCharacteristic( rChar(r) );
+    if (r->cf->extRing->qideal!=NULL)
+    {
+      CanonicalForm mipo=convSingPFactoryP(r->cf->extRing->qideal->m[0],
+                                                 r->cf->extRing);
+      Variable a=rootOf(mipo);
+      CanonicalForm F( convSingAPFactoryAP( f,a,r ) ),
+                    G( convSingAPFactoryAP( g,a,r ) );
+      res= convFactoryAPSingAP(  F * G, r  );
+      prune (a);
+    }
+    else
+    {
+      CanonicalForm F( convSingTrPFactoryP( f,r ) ), G( convSingTrPFactoryP( g,r ) );
+      res= convFactoryPSingTrP(  F * G,r  );
+    }
+  }
+#if 0 // not yet working
+  else if (rField_is_GF())
+  {
+    //Print("GF(%d^%d)\n",nfCharP,nfMinPoly[0]);
+    setCharacteristic( nfCharP,nfMinPoly[0], currRing->parameter[0][0] );
+    CanonicalForm F( convSingGFFactoryGF( f ) ), G( convSingGFFactoryGF( g ) );
+    res = convFactoryGFSingGF( F * G );
+  }
+#endif
+  else
+    WerrorS( feNotImplemented );
+  Off(SW_RATIONAL);
+  return res;
+}
+
+poly singclap_pdivide ( poly f, poly g, const ring r )
+{
+  poly res=NULL;
+
+  #ifdef HAVE_FLINT
+  #if __FLINT_RELEASE >= 20503
+  /*
+    If the division is not exact, control will pass to factory where the
+    polynomials can be divided using the ordering that factory chooses.
+  */
+  if (rField_is_Zp(r))
+  {
+    nmod_mpoly_ctx_t ctx;
+    if (!convSingRFlintR(ctx,r))
+    {
+      res = Flint_Divide_MP(f,0,g,0,ctx,r);
+      if (res != NULL)
+        return res;
+    }
+  }
+  else
+  if (rField_is_Q(r))
+  {
+    fmpq_mpoly_ctx_t ctx;
+    if (!convSingRFlintR(ctx,r))
+    {
+      res = Flint_Divide_MP(f,0,g,0,ctx,r);
+      if (res != NULL)
+        return res;
+    }
+  }
+  #endif
+  #endif
+
+  On(SW_RATIONAL);
+  if (rField_is_Zp(r) || rField_is_Q(r)
+  || (rField_is_Zn(r)&&(r->cf->convSingNFactoryN!=ndConvSingNFactoryN)))
   {
     setCharacteristic( rChar(r) );
     CanonicalForm F( convSingPFactoryP( f,r ) ), G( convSingPFactoryP( g,r ) );
     res = convFactoryPSingP( F / G,r );
   }
-  else if (rField_is_Ring_Z(r))
-  {
-    Off(SW_RATIONAL);
-    setCharacteristic( rChar(r) );
-    CanonicalForm F( convSingPFactoryP( f,r ) ), G( convSingPFactoryP( g,r ) );
-    res = convFactoryPSingP( F / G,r );
-  }
+  // div is not implemented for ZZ coeffs in factory
   else if (r->cf->extRing!=NULL)
   {
     if (rField_is_Q_a(r)) setCharacteristic( 0 );
@@ -596,6 +665,64 @@ poly singclap_pdivide ( poly f, poly g, const ring r )
   return res;
 }
 
+poly singclap_pmod ( poly f, poly g, const ring r )
+{
+  poly res=NULL;
+  On(SW_RATIONAL);
+  if (rField_is_Zp(r) || rField_is_Q(r)
+  || (rField_is_Zn(r)&&(r->cf->convSingNFactoryN!=ndConvSingNFactoryN)))
+  {
+    setCharacteristic( rChar(r) );
+    CanonicalForm F( convSingPFactoryP( f,r ) ), G( convSingPFactoryP( g,r ) );
+    CanonicalForm Q,R;
+    divrem(F,G,Q,R);
+    res = convFactoryPSingP(R,r);
+    //res = convFactoryPSingP( F-(F/G)*G,r );
+  }
+  // mod is not implemented for ZZ coeffs in factory
+  else if (r->cf->extRing!=NULL)
+  {
+    if (rField_is_Q_a(r)) setCharacteristic( 0 );
+    else               setCharacteristic( rChar(r) );
+    if (r->cf->extRing->qideal!=NULL)
+    {
+      CanonicalForm mipo=convSingPFactoryP(r->cf->extRing->qideal->m[0],
+                                                 r->cf->extRing);
+      Variable a=rootOf(mipo);
+      CanonicalForm F( convSingAPFactoryAP( f,a,r ) ),
+                    G( convSingAPFactoryAP( g,a,r ) );
+      CanonicalForm Q,R;
+      divrem(F,G,Q,R);
+      res = convFactoryAPSingAP(R,r);
+      //res= convFactoryAPSingAP( F-(F/G)*G, r  );
+      prune (a);
+    }
+    else
+    {
+      CanonicalForm F( convSingTrPFactoryP( f,r ) ), G( convSingTrPFactoryP( g,r ) );
+      CanonicalForm Q,R;
+      divrem(F,G,Q,R);
+      res = convFactoryPSingTrP(R,r);
+      //res= convFactoryPSingTrP(  F-(F/G)*G,r  );
+    }
+  }
+#if 0 // not yet working
+  else if (rField_is_GF())
+  {
+    //Print("GF(%d^%d)\n",nfCharP,nfMinPoly[0]);
+    setCharacteristic( nfCharP,nfMinPoly[0], currRing->parameter[0][0] );
+    CanonicalForm F( convSingGFFactoryGF( f ) ), G( convSingGFFactoryGF( g ) );
+    res = convFactoryGFSingGF( F / G );
+  }
+#endif
+  else
+    WerrorS( feNotImplemented );
+  Off(SW_RATIONAL);
+  return res;
+}
+
+#if 0
+// unused
 void singclap_divide_content ( poly f, const ring r )
 {
   if ( f==NULL )
@@ -692,13 +819,14 @@ void singclap_divide_content ( poly f, const ring r )
     // pTest(f);
   }
 }
+#endif
 
 BOOLEAN count_Factors(ideal I, intvec *v,int j, poly &f, poly fac, const ring r)
 {
   p_Test(f,r);
   p_Test(fac,r);
   int e=0;
-  if (!p_IsConstantPoly(fac,r))
+  if (!p_IsConstant(fac,r))
   {
 #ifdef FACTORIZE2_DEBUG
     printf("start count_Factors(%d), Fdeg=%d, factor deg=%d\n",j,p_Totaldegree(f,r),p_Totaldegree(fac,r));
@@ -707,7 +835,8 @@ BOOLEAN count_Factors(ideal I, intvec *v,int j, poly &f, poly fac, const ring r)
     On(SW_RATIONAL);
     CanonicalForm F, FAC,Q,R;
     Variable a;
-    if (rField_is_Zp(r) || rField_is_Q(r))
+    if (rField_is_Zp(r) || rField_is_Q(r)
+    || (rField_is_Zn(r)&&(r->cf->convSingNFactoryN!=ndConvSingNFactoryN)))
     {
       F=convSingPFactoryP( f,r );
       FAC=convSingPFactoryP( fac,r );
@@ -741,7 +870,8 @@ BOOLEAN count_Factors(ideal I, intvec *v,int j, poly &f, poly fac, const ring r)
       R-=F;
       if (R.isZero())
       {
-        if (rField_is_Zp(r) || rField_is_Q(r))
+        if (rField_is_Zp(r) || rField_is_Q(r)
+        || (rField_is_Zn(r)&&(r->cf->convSingNFactoryN!=ndConvSingNFactoryN)))
         {
           q = convFactoryPSingP( Q,r );
         }
@@ -779,7 +909,7 @@ BOOLEAN count_Factors(ideal I, intvec *v,int j, poly &f, poly fac, const ring r)
   return TRUE;
 }
 
-int singclap_factorize_retry;
+VAR int singclap_factorize_retry;
 
 ideal singclap_factorize ( poly f, intvec ** v , int with_exps, const ring r)
 /* destroys f, sets *v */
@@ -831,7 +961,7 @@ ideal singclap_factorize ( poly f, intvec ** v , int with_exps, const ring r)
     }
     if (n==0)
     {
-      res->m[0]=p_One(r);
+      if (res->m[0]==NULL) res->m[0]=p_One(r);
       // (**v)[0]=1; is already done
     }
     else
@@ -855,6 +985,7 @@ ideal singclap_factorize ( poly f, intvec ** v , int with_exps, const ring r)
   }
   //PrintS("S:");p_Write(f,r);PrintLn();
   // use factory/libfac in general ==============================
+  Variable dummy(-1); prune(dummy); // remove all (tmp.) extensions
   Off(SW_RATIONAL);
   On(SW_SYMMETRIC_FF);
   CFFList L;
@@ -863,81 +994,88 @@ ideal singclap_factorize ( poly f, intvec ** v , int with_exps, const ring r)
   number old_lead_coeff=n_Copy(pGetCoeff(f), r->cf);
 
   Variable a;
-  if (!rField_is_Zp(r) && !rField_is_Zp_a(r) && !rField_is_Z(r)) /* Q, Q(a) */
+  if (r->cf->convSingNFactoryN!=ndConvSingNFactoryN)
   {
-    //if (f!=NULL) // already tested at start of routine
+    if (rField_is_Q(r) || rField_is_Q_a(r) || rField_is_Z(r)) /* Q, Q(a), Z */
     {
-      number n0=n_Copy(pGetCoeff(f),r->cf);
-      if (with_exps==0)
-        N=n_Copy(n0,r->cf);
-      p_Cleardenom(f, r);
-      //after here f should not have a denominator!!
-      //PrintS("S:");p_Write(f,r);PrintLn();
-      NN=n_Div(n0,pGetCoeff(f),r->cf);
-      n_Delete(&n0,r->cf);
-      if (with_exps==0)
+      //if (f!=NULL) // already tested at start of routine
       {
-        n_Delete(&N,r->cf);
-        N=n_Copy(NN,r->cf);
+        number n0=n_Copy(pGetCoeff(f),r->cf);
+        if (with_exps==0)
+          N=n_Copy(n0,r->cf);
+        if (rField_is_Z(r)) p_Content(f, r);
+        else                p_Cleardenom(f, r);
+        //after here f should not have a denominator!! and no content
+        //PrintS("S:");p_Write(f,r);PrintLn();
+        NN=n_Div(n0,pGetCoeff(f),r->cf);
+        n_Delete(&n0,r->cf);
+        if (with_exps==0)
+        {
+          n_Delete(&N,r->cf);
+          N=n_Copy(NN,r->cf);
+        }
       }
     }
-  }
-  else if (rField_is_Zp_a(r))
-  {
-    //if (f!=NULL) // already tested at start of routine
-    if (singclap_factorize_retry==0)
+    else if (rField_is_Zp_a(r))
     {
-      number n0=n_Copy(pGetCoeff(f),r->cf);
-      if (with_exps==0)
-        N=n_Copy(n0,r->cf);
-      p_Norm(f,r);
-      p_Cleardenom(f, r);
-      NN=n_Div(n0,pGetCoeff(f),r->cf);
-      n_Delete(&n0,r->cf);
-      if (with_exps==0)
+      //if (f!=NULL) // already tested at start of routine
+      if (singclap_factorize_retry==0)
       {
-        n_Delete(&N,r->cf);
-        N=n_Copy(NN,r->cf);
+        number n0=n_Copy(pGetCoeff(f),r->cf);
+        if (with_exps==0)
+          N=n_Copy(n0,r->cf);
+        p_Norm(f,r);
+        p_Cleardenom(f, r);
+        NN=n_Div(n0,pGetCoeff(f),r->cf);
+        n_Delete(&n0,r->cf);
+        if (with_exps==0)
+        {
+          n_Delete(&N,r->cf);
+          N=n_Copy(NN,r->cf);
+        }
       }
     }
-  }
-  if (rField_is_Q(r) || rField_is_Zp(r) || (rField_is_Z(r)))
-  {
-    setCharacteristic( rChar(r) );
-    CanonicalForm F( convSingPFactoryP( f,r ) );
-    L = factorize( F );
-  }
-  // and over Q(a) / Fp(a)
-  else if (r->cf->extRing!=NULL)
-  {
-    if (rField_is_Q_a (r)) setCharacteristic (0);
-    else                   setCharacteristic( rChar(r) );
-    if (r->cf->extRing->qideal!=NULL)
+    if (rField_is_Q(r) || rField_is_Zp(r) || rField_is_Z(r) || rField_is_Zn(r))
     {
-      CanonicalForm mipo=convSingPFactoryP(r->cf->extRing->qideal->m[0],
-                                           r->cf->extRing);
-      a=rootOf(mipo);
-      CanonicalForm F( convSingAPFactoryAP( f, a, r ) );
-      if (rField_is_Zp_a(r))
+      setCharacteristic( rChar(r) );
+      if (errorreported) goto notImpl; // char too large
+      CanonicalForm F( convSingPFactoryP( f,r ) );
+      L = factorize( F );
+    }
+    // and over Q(a) / Fp(a)
+    else if (r->cf->extRing!=NULL)
+    {
+      if (rField_is_Q_a (r)) setCharacteristic (0);
+      else                   setCharacteristic( rChar(r) );
+      if (errorreported) goto notImpl; // char too large
+      if (r->cf->extRing->qideal!=NULL) /*algebraic extension */
       {
+        CanonicalForm mipo=convSingPFactoryP(r->cf->extRing->qideal->m[0],
+                                             r->cf->extRing);
+        a=rootOf(mipo);
+        CanonicalForm F( convSingAPFactoryAP( f, a, r ) );
         L = factorize( F, a );
+        prune(a);
       }
-      else
+      else /* rational functions */
       {
-        //  over Q(a)
-        L= factorize (F, a);
+        CanonicalForm F( convSingTrPFactoryP( f,r ) );
+        L = factorize( F );
       }
     }
     else
     {
-      CanonicalForm F( convSingTrPFactoryP( f,r ) );
-      L = factorize( F );
+      goto notImpl;
     }
   }
   else
   {
     goto notImpl;
   }
+  if (errorreported)
+  {
+    errorreported=FALSE;
+  }
   {
     poly ff=p_Copy(f,r); // a copy for the retry stuff
     // the first factor should be a constant
@@ -961,7 +1099,8 @@ ideal singclap_factorize ( poly f, intvec ** v , int with_exps, const ring r)
     for ( ; J.hasItem(); J++, j++ )
     {
       if (with_exps!=1) (**v)[j] = J.getItem().exp();
-      if (rField_is_Zp(r) || rField_is_Q(r)||  rField_is_Z(r))           /* Q, Fp, Z */
+      if (rField_is_Zp(r) || rField_is_Q(r)||  rField_is_Z(r)
+      || rField_is_Zn(r))           /* Q, Fp, Z */
       {
         //count_Factors(res,*v,f, j, convFactoryPSingP( J.getItem().factor() );
         res->m[j] = convFactoryPSingP( J.getItem().factor(),r );
@@ -1008,7 +1147,7 @@ ideal singclap_factorize ( poly f, intvec ** v , int with_exps, const ring r)
       if (r->cf->extRing->qideal!=NULL)
         prune (a);
 #ifndef SING_NDEBUG
-    if ((r->cf->extRing!=NULL) && (!p_IsConstantPoly(ff,r)))
+    if ((r->cf->extRing!=NULL) && (!p_IsConstant(ff,r)))
     {
       singclap_factorize_retry++;
       if (singclap_factorize_retry<3)
@@ -1060,7 +1199,7 @@ ideal singclap_factorize ( poly f, intvec ** v , int with_exps, const ring r)
     p_Delete(&ff,r);
     if (N!=NULL)
     {
-      p_Mult_nn(res->m[0],N,r);
+      __p_Mult_nn(res->m[0],N,r);
       n_Delete(&N,r->cf);
       N=NULL;
     }
@@ -1137,8 +1276,20 @@ ideal singclap_factorize ( poly f, intvec ** v , int with_exps, const ring r)
     n_Delete(&old_lead_coeff,r->cf);
   errorreported=save_errorreported;
 notImpl:
+  prune(a);
   if (res==NULL)
+  {
     WerrorS( feNotImplemented );
+    if ((v!=NULL) && ((*v)!=NULL) &&(with_exps==2))
+    {
+       *v = new intvec( 1 );
+       (*v)[0]=1;
+    }
+    res=idInit(2,1);
+    res->m[0]=p_One(r);
+    res->m[1]=f;
+  }
+  else p_Delete(&f,r);
   if (NN!=NULL)
   {
     n_Delete(&NN,r->cf);
@@ -1147,8 +1298,6 @@ notImpl:
   {
     n_Delete(&N,r->cf);
   }
-  if (f!=NULL) p_Delete(&f,r);
-  //PrintS("......S\n");
   return res;
 }
 
@@ -1185,22 +1334,10 @@ ideal singclap_sqrfree ( poly f, intvec ** v , int with_exps, const ring r)
     for(i=rVar(r);i>0;i--) if(p_GetExp(f,i,r)!=0) n++;
     if (with_exps==0 || with_exps==3) n++; // with coeff
     res=idInit(si_max(n,1),1);
-    switch(with_exps)
+    if(with_exps!=1)
     {
-      case 0: // with coef & exp.
-        res->m[0]=p_NSet(n_Copy(pGetCoeff(f),r->cf),r);
-        // no break
-      case 3: // with coef & exp.
-        res->m[0]=p_NSet(n_Copy(pGetCoeff(f),r->cf),r);
-        // no break
-      case 2: // with exp.
         (*v)=new intvec(si_max(1,n));
         (**v)[0]=1;
-        // no break
-      case 1: ;
-      #ifdef TEST
-      default: ;
-      #endif
     }
     res->m[0]=p_NSet(n_Copy(pGetCoeff(f),r->cf),r);
     if (n==0)
@@ -1240,7 +1377,7 @@ ideal singclap_sqrfree ( poly f, intvec ** v , int with_exps, const ring r)
   if (!rField_is_Zp(r) && !rField_is_Zp_a(r)) /* Q, Q(a) */
   {
     //if (f!=NULL) // already tested at start of routine
-    number n0=n_Copy(pGetCoeff(f),r->cf);
+    number n0=n_Copy(old_lead_coeff,r->cf);
     if (with_exps==0 || with_exps==3)
       N=n_Copy(n0,r->cf);
     p_Cleardenom(f, r);
@@ -1259,7 +1396,7 @@ ideal singclap_sqrfree ( poly f, intvec ** v , int with_exps, const ring r)
     //if (f!=NULL) // already tested at start of routine
     if (singclap_factorize_retry==0)
     {
-      number n0=n_Copy(pGetCoeff(f),r->cf);
+      number n0=n_Copy(old_lead_coeff,r->cf);
       if (with_exps==0 || with_exps==3)
         N=n_Copy(n0,r->cf);
       p_Norm(f,r);
@@ -1273,7 +1410,8 @@ ideal singclap_sqrfree ( poly f, intvec ** v , int with_exps, const ring r)
       }
     }
   }
-  if (rField_is_Q(r) || rField_is_Zp(r))
+  if (rField_is_Q(r) || rField_is_Zp(r)
+  || (rField_is_Zn(r)&&(r->cf->convSingNFactoryN!=ndConvSingNFactoryN)))
   {
     setCharacteristic( rChar(r) );
     CanonicalForm F( convSingPFactoryP( f,r ) );
@@ -1341,7 +1479,8 @@ ideal singclap_sqrfree ( poly f, intvec ** v , int with_exps, const ring r)
     for ( ; J.hasItem(); J++, j++ )
     {
       if (with_exps!=1 && with_exps!=3) (**v)[j] = J.getItem().exp();
-      if (rField_is_Zp(r) || rField_is_Q(r))
+      if (rField_is_Zp(r) || rField_is_Q(r)
+      || (rField_is_Zn(r)&&(r->cf->convSingNFactoryN!=ndConvSingNFactoryN)))
         res->m[j] = convFactoryPSingP( J.getItem().factor(),r );
       else if (r->cf->extRing!=NULL)     /* Q(a), Fp(a) */
       {
@@ -1357,7 +1496,7 @@ ideal singclap_sqrfree ( poly f, intvec ** v , int with_exps, const ring r)
     }
     if (N!=NULL)
     {
-      p_Mult_nn(res->m[0],N,r);
+      __p_Mult_nn(res->m[0],N,r);
       n_Delete(&N,r->cf);
       N=NULL;
     }
@@ -1407,7 +1546,8 @@ matrix singclap_irrCharSeries ( ideal I, const ring r)
   On(SW_SYMMETRIC_FF);
   CFList L;
   ListCFList LL;
-  if (rField_is_Q(r) || rField_is_Zp(r))
+  if (rField_is_Q(r) || rField_is_Zp(r)
+  || (rField_is_Zn(r)&&(r->cf->convSingNFactoryN!=ndConvSingNFactoryN)))
   {
     setCharacteristic( rChar(r) );
     for(i=0;i<IDELEMS(I);i++)
@@ -1418,6 +1558,7 @@ matrix singclap_irrCharSeries ( ideal I, const ring r)
         p=p_Copy(p,r);
         p_Cleardenom(p, r);
         L.append(convSingPFactoryP(p,r));
+        p_Delete(&p,r);
       }
     }
   }
@@ -1433,6 +1574,7 @@ matrix singclap_irrCharSeries ( ideal I, const ring r)
         p=p_Copy(p,r);
         p_Cleardenom(p, r);
         L.append(convSingTrPFactoryP(p,r));
+        p_Delete(&p,r);
       }
     }
   }
@@ -1474,7 +1616,8 @@ matrix singclap_irrCharSeries ( ideal I, const ring r)
   {
     for (n=1, Li = LLi.getItem(); Li.hasItem(); Li++, n++)
     {
-      if (rField_is_Q(r) || rField_is_Zp(r))
+      if (rField_is_Q(r) || rField_is_Zp(r)
+      || (rField_is_Zn(r)&&(r->cf->convSingNFactoryN!=ndConvSingNFactoryN)))
         MATELEM(res,m,n)=convFactoryPSingP(Li.getItem(),r);
       else
         MATELEM(res,m,n)=convFactoryPSingTrP(Li.getItem(),r);
@@ -1490,7 +1633,8 @@ char* singclap_neworder ( ideal I, const ring r)
   Off(SW_RATIONAL);
   On(SW_SYMMETRIC_FF);
   CFList L;
-  if (rField_is_Q(r) || rField_is_Zp(r))
+  if (rField_is_Q(r) || rField_is_Zp(r)
+  || (rField_is_Zn(r)&&(r->cf->convSingNFactoryN!=ndConvSingNFactoryN)))
   {
     setCharacteristic( rChar(r) );
     for(i=0;i<IDELEMS(I);i++)
@@ -1635,7 +1779,7 @@ number singclap_det_bi( bigintmat * m, const coeffs cf)
   return res;
 }
 
-#ifdef HAVE_NTL
+#if defined(HAVE_NTL) || defined(AHVE_FLINT)
 matrix singntl_HNF(matrix  m, const ring s )
 {
   int r=m->rows();
@@ -1645,7 +1789,7 @@ matrix singntl_HNF(matrix  m, const ring s )
     return NULL;
   }
 
-  matrix res=mp_New(r,r);
+  matrix res=mpNew(r,r);
 
   if (rField_is_Q(s))
   {
@@ -1738,7 +1882,7 @@ matrix singntl_LLL(matrix  m, const ring s )
 {
   int r=m->rows();
   int c=m->cols();
-  matrix res=mp_New(r,c);
+  matrix res=mpNew(r,c);
   if (rField_is_Q(s))
   {
     CFMatrix M(r,c);
@@ -1789,7 +1933,33 @@ intvec* singntl_LLL(intvec*  m)
   delete MM;
   return mm;
 }
+#else
+matrix singntl_HNF(matrix  m, const ring s )
+{
+  WerrorS("NTL/FLINT missing");
+  return NULL;
+}
 
+intvec* singntl_HNF(intvec*  m)
+{
+  WerrorS("NTL/FLINT missing");
+  return NULL;
+}
+
+matrix singntl_LLL(matrix  m, const ring s )
+{
+  WerrorS("NTL/FLINT missing");
+  return NULL;
+}
+
+intvec* singntl_LLL(intvec*  m)
+{
+  WerrorS("NTL/FLINT missing");
+  return NULL;
+}
+#endif
+
+#ifdef HAVE_NTL
 ideal singclap_absFactorize ( poly f, ideal & mipos, intvec ** exps, int & numFactors, const ring r)
 {
   p_Test(f, r);
@@ -1867,30 +2037,6 @@ ideal singclap_absFactorize ( poly f, ideal & mipos, intvec ** exps, int & numFa
 }
 
 #else
-matrix singntl_HNF(matrix  m, const ring s )
-{
-  WerrorS("NTL missing");
-  return NULL;
-}
-
-intvec* singntl_HNF(intvec*  m)
-{
-  WerrorS("NTL missing");
-  return NULL;
-}
-
-matrix singntl_LLL(matrix  m, const ring s )
-{
-  WerrorS("NTL missing");
-  return NULL;
-}
-
-intvec* singntl_LLL(intvec*  m)
-{
-  WerrorS("NTL missing");
-  return NULL;
-}
-
 ideal singclap_absFactorize ( poly f, ideal & mipos, intvec ** exps, int & numFactors, const ring r)
 {
   WerrorS("NTL missing");
@@ -1899,3 +2045,8 @@ ideal singclap_absFactorize ( poly f, ideal & mipos, intvec ** exps, int & numFa
 
 #endif /* HAVE_NTL */
 
+int * Zp_roots(poly p, const ring r)
+{
+  CanonicalForm pp=convSingPFactoryP(p,r);
+  return Zp_roots(pp);
+}
