@@ -28,34 +28,27 @@
   *               computing in K[a_1, ..., a_s] / I.
   **/
 
+#include "misc/auxiliary.h"
 
+#include "reporter/reporter.h"
 
+#include "coeffs/coeffs.h"
+#include "coeffs/numbers.h"
+#include "coeffs/longrat.h"
 
-#include <misc/auxiliary.h>
+#include "polys/monomials/ring.h"
+#include "polys/monomials/p_polys.h"
+#include "polys/simpleideals.h"
+#include "polys/PolyEnumerator.h"
 
-#include <omalloc/omalloc.h>
+#include "factory/factory.h"
+#include "polys/clapconv.h"
+#include "polys/clapsing.h"
+#include "polys/prCopy.h"
 
-#include <reporter/reporter.h>
-
-#include <coeffs/coeffs.h>
-#include <coeffs/numbers.h>
-
-#include <coeffs/longrat.h>
-
-#include <polys/monomials/ring.h>
-#include <polys/monomials/p_polys.h>
-#include <polys/simpleideals.h>
-
-#include <polys/PolyEnumerator.h>
-
-#include <factory/factory.h>
-#include <polys/clapconv.h>
-#include <polys/clapsing.h>
-#include <polys/prCopy.h>
-
-#include <polys/ext_fields/algext.h>
+#include "polys/ext_fields/algext.h"
 #define TRANSEXT_PRIVATES 1
-#include <polys/ext_fields/transext.h>
+#include "polys/ext_fields/transext.h"
 
 #ifdef LDEBUG
 #define naTest(a) naDBTest(a,__FILE__,__LINE__,cf)
@@ -368,11 +361,11 @@ BOOLEAN naGreater(number a, number b, const coeffs cf)
   if (naIsZero(a, cf))
   {
     if (naIsZero(b, cf)) return FALSE;
-    return !n_GreaterZero(pGetCoeff((poly)b),cf);
+    return !n_GreaterZero(pGetCoeff((poly)b),naCoeffs);
   }
   if (naIsZero(b, cf))
   {
-    return n_GreaterZero(pGetCoeff((poly)a),cf);
+    return n_GreaterZero(pGetCoeff((poly)a),naCoeffs);
   }
   int aDeg = p_Totaldegree((poly)a, naRing);
   int bDeg = p_Totaldegree((poly)b, naRing);
@@ -407,12 +400,15 @@ void naCoeffWrite(const coeffs cf, BOOLEAN details)
   const int P = rVar(A);
   assume( P > 0 );
 
-  Print("//   %d parameter    : ", P);
+  PrintS("[");
 
   for (int nop=0; nop < P; nop ++)
-    Print("%s ", rRingVar(nop, A));
+  {
+    Print("%s", rRingVar(nop, A));
+    if (nop!=P-1) PrintS(", ");
+  }
 
-  PrintLn();
+  PrintS("]/(");
 
   const ideal I = A->qideal;
 
@@ -422,14 +418,11 @@ void naCoeffWrite(const coeffs cf, BOOLEAN details)
 
   if ( details )
   {
-    PrintS("//   minpoly        : (");
     p_Write0( I->m[0], A);
     PrintS(")");
   }
   else
-    PrintS("//   minpoly        : ...");
-
-  PrintLn();
+    PrintS("...)");
 
 /*
   char *x = rRingVar(0, A);
@@ -467,8 +460,7 @@ number naMult(number a, number b, const coeffs cf)
 {
   naTest(a); naTest(b);
   if ((a == NULL)||(b == NULL)) return NULL;
-  poly aTimesB = p_Mult_q(p_Copy((poly)a, naRing),
-                          p_Copy((poly)b, naRing), naRing);
+  poly aTimesB = pp_Mult_qq((poly)a, (poly)b, naRing);
   definiteReduce(aTimesB, naMinpoly, cf);
   p_Normalize(aTimesB,naRing);
   return (number)aTimesB;
@@ -627,8 +619,7 @@ number naLcm(number a, number b, const coeffs cf)
   naTest(a); naTest(b);
   if (a == NULL) return NULL;
   if (b == NULL) return NULL;
-  number theProduct = (number)p_Mult_q(p_Copy((poly)a, naRing),
-                                       p_Copy((poly)b, naRing), naRing);
+  number theProduct = (number)pp_Mult_qq((poly)a, (poly)b, naRing);
   /* note that theProduct needs not be reduced w.r.t. naMinpoly;
      but the final division will take care of the necessary reduction */
   number theGcd = naGcd(a, b, cf);
@@ -720,11 +711,7 @@ static BOOLEAN naCoeffIsEqual(const coeffs cf, n_coeffType n, void * param)
 
 int naSize(number a, const coeffs cf)
 {
-  if (a == NULL) return -1;
-  /* this has been taken from the old implementation of field extensions,
-     where we computed the sum of the degree and the number of terms in
-     (poly)a; so we leave it at that, for the time being;
-     maybe, the number of terms alone is a better measure? */
+  if (a == NULL) return 0;
   poly aAsPoly = (poly)a;
   int theDegree = 0; int noOfTerms = 0;
   while (aAsPoly != NULL)
@@ -734,7 +721,7 @@ int naSize(number a, const coeffs cf)
     if (d > theDegree) theDegree = d;
     pIter(aAsPoly);
   }
-  return theDegree + noOfTerms;
+  return (theDegree +1) * noOfTerms;
 }
 
 /* performs polynomial division and overrides p by the remainder
@@ -823,7 +810,7 @@ number naGcd(number a, number b, const coeffs cf)
 #if 0
   naTest(a); naTest(b);
   const ring R = naRing;
-  return (number) singclap_gcd(p_Copy((poly)a, R), p_Copy((poly)b, R), R);
+  return (number) singclap_gcd_r((poly)a, (poly)b, R);
 #endif
 //  return (number)p_Gcd((poly)a, (poly)b, naRing);
 }
@@ -1032,8 +1019,6 @@ nMapFunc naSetMap(const coeffs src, const coeffs dst)
   /* dst is expected to be an algebraic field extension */
   assume(getCoeffType(dst) == n_algExt);
 
-  if( src == dst ) return ndCopyMap;
-
   int h = 0; /* the height of the extension tower given by dst */
   coeffs bDst = nCoeff_bottom(dst, h); /* the bottom field in the tower dst */
   coeffs bSrc = nCoeff_bottom(src, h); /* the bottom field in the tower src */
@@ -1114,7 +1099,7 @@ int naIsParam(number m, const coeffs cf)
 }
 
 
-void naClearContent(ICoeffsEnumerator& numberCollectionEnumerator, number& c, const coeffs cf)
+static void naClearContent(ICoeffsEnumerator& numberCollectionEnumerator, number& c, const coeffs cf)
 {
   assume(cf != NULL);
   assume(getCoeffType(cf) == n_algExt);
@@ -1182,7 +1167,11 @@ void naClearContent(ICoeffsEnumerator& numberCollectionEnumerator, number& c, co
 
 //    p_InpGcd(cand, (poly)n, R);
 
-    cand = singclap_gcd(cand, p_Copy((poly)n, R), R);
+    { // R->cf is QQ
+      poly tmp=gcd_over_Q(cand,(poly)n,R);
+      p_Delete(&cand,R);
+      cand=tmp;
+    }
 
 //    cand1 = p_Gcd(cand,(poly)n, R); p_Delete(&cand, R); cand = cand1;
 
@@ -1256,7 +1245,7 @@ void naClearContent(ICoeffsEnumerator& numberCollectionEnumerator, number& c, co
   n_ClearContent(itr, cc, Q); // TODO: get rid of (-LC) normalization!?
 
   // over alg. ext. of Q // takes over the input number
-  c = (number) p_Mult_nn( (poly)c, cc, R);
+  c = (number) __p_Mult_nn( (poly)c, cc, R);
 //      p_Mult_q(p_NSet(cc, R), , R);
 
   n_Delete(&cc, Q);
@@ -1337,29 +1326,6 @@ void naKillChar(coeffs cf)
      rDelete(cf->extRing);
 }
 
-char* naCoeffString(const coeffs r) // currently also for tranext.
-{
-  const char* const* p=n_ParameterNames(r);
-  int l=0;
-  int i;
-  for(i=0; i<n_NumberOfParameters(r);i++)
-  {
-    l+=(strlen(p[i])+1);
-  }
-  char *s=(char *)omAlloc(l+10+1);
-  s[0]='\0';
-  snprintf(s,10+1,"%d",r->ch); /* Fp(a) or Q(a) */
-  char tt[2];
-  tt[0]=',';
-  tt[1]='\0';
-  for(i=0; i<n_NumberOfParameters(r);i++)
-  {
-    strcat(s,tt);
-    strcat(s,p[i]);
-  }
-  return s;
-}
-
 char* naCoeffName(const coeffs r) // currently also for tranext.
 {
   const char* const* p=n_ParameterNames(r);
@@ -1369,7 +1335,7 @@ char* naCoeffName(const coeffs r) // currently also for tranext.
   {
     l+=(strlen(p[i])+1);
   }
-  static char s[200];
+  STATIC_VAR char s[200];
   s[0]='\0';
   snprintf(s,10+1,"%d",r->ch); /* Fp(a) or Q(a) */
   char tt[2];
@@ -1436,7 +1402,6 @@ BOOLEAN naInitChar(coeffs cf, void * infoStruct)
   p_Test((poly)naMinpoly, naRing);
   #endif
 
-  cf->cfCoeffString  = naCoeffString;
   cf->cfCoeffName    = naCoeffName;
 
   cf->cfGreaterZero  = naGreaterZero;
@@ -1554,8 +1519,7 @@ number n2pMult(number a, number b, const coeffs cf)
 {
   n2pTest(a); n2pTest(b);
   if ((a == NULL)||(b == NULL)) return NULL;
-  poly aTimesB = p_Mult_q(p_Copy((poly)a, n2pRing),
-                          p_Copy((poly)b, n2pRing), n2pRing);
+  poly aTimesB = pp_Mult_qq((poly)a, (poly)b, n2pRing);
   return (number)aTimesB;
 }
 
@@ -1604,34 +1568,6 @@ static BOOLEAN n2pCoeffIsEqual(const coeffs cf, n_coeffType n, void * param)
   return FALSE;
 }
 
-char* n2pCoeffString(const coeffs cf)
-{
-  const char* const* p=n_ParameterNames(cf);
-  int l=0;
-  int i;
-  for(i=0; i<rVar(n2pRing);i++)
-  {
-    l+=(strlen(p[i])+1);
-  }
-  char *cf_s=nCoeffString(n2pRing->cf);
-  char *s=(char *)omAlloc(l+5+strlen(cf_s));
-  s[0]='\0';
-  snprintf(s,strlen(cf_s)+2,"%s",cf_s);
-  omFree(cf_s);
-  char tt[2];
-  tt[0]='[';
-  tt[1]='\0';
-  strcat(s,tt);
-  tt[0]=',';
-  for(i=0; i<rVar(n2pRing);i++)
-  {
-    strcat(s,p[i]);
-    if (i+1!=rVar(n2pRing)) strcat(s,tt);
-    else { tt[0]=']'; strcat(s,tt); }
-  }
-  return s;
-}
-
 char* n2pCoeffName(const coeffs cf)
 {
   const char* const* p=n_ParameterNames(cf);
@@ -1641,11 +1577,10 @@ char* n2pCoeffName(const coeffs cf)
   {
     l+=(strlen(p[i])+1);
   }
-  char *cf_s=nCoeffString(n2pRing->cf);
-  static char s[200];
+  char *cf_s=nCoeffName(n2pRing->cf);
+  STATIC_VAR char s[200];
   s[0]='\0';
   snprintf(s,strlen(cf_s)+2,"%s",cf_s);
-  omFree(cf_s);
   char tt[2];
   tt[0]='[';
   tt[1]='\0';
@@ -1710,7 +1645,6 @@ BOOLEAN n2pInitChar(coeffs cf, void * infoStruct)
   cf->is_field=FALSE;
   cf->is_domain=TRUE;
 
-  cf->cfCoeffString  = n2pCoeffString;
   cf->cfCoeffName    = n2pCoeffName;
 
   cf->cfGreaterZero  = naGreaterZero;
