@@ -1,7 +1,18 @@
-#!/bin/sh
-TARVERSION=4.1.1
-VERSION=4.1.1
-export VERSION TARVERSION
+#!/usr/bin/env bash
+# Hannes helper to build tar files for the ftp server
+# for the patch-versions (version between official releases x.x.x)
+# for official versions: use "make dist"/"make distcheck"
+
+set -e
+
+TARVERSION=4.2.0
+VERSION=4.2.0
+BUILD_DIR=/tmp2/wawa
+export VERSION TARVERSION BUILD_DIR
+
+# sanity check
+if test -e $BUILD_DIR/singularconfig.h
+then
 
 git archive --prefix=singular-$VERSION/ HEAD |tar xf -
 mkdir singular-$VERSION/doc
@@ -14,8 +25,6 @@ fi
 
 command rm singular-$VERSION/.gdbinit singular-$VERSION/*/.gdbinit singular-$VERSION/*/*/.gdbinit singular-$VERSION/*/*/*/.gdbinit
 command rm singular-$VERSION/IntegerProgramming/README
-command rm -rf singular-$VERSION/modules
-command rm -rf singular-$VERSION/factory/old
 cd singular-$VERSION
 ./autogen.sh
 cd ..
@@ -29,8 +38,15 @@ command rm -rf singular-$VERSION/kernel/old singular-$VERSION/kernel/sample
 command rm -rf singular-$VERSION/dox/Doxyfile.html.all singular-$VERSION/dox/Doxyfile.in1 singular-$VERSION/dox/Doxyfile.latex.short singular-$VERSION/dox/readme
 command rm singular-$VERSION/factory/ConwayList.txt
 command rm -rf singular-$VERSION/logo  singular-$VERSION/standalone.test  singular-$VERSION/templates  singular-$VERSION/tests
-cp redhat/singular.spec singular-$VERSION/redhat/singular.spec
-cp /tmp2/wawa-i/share/singular/LIB/all.lib  singular-$VERSION/Singular/LIB/.
+cp $BUILD_DIR/redhat/singular.spec singular-$VERSION/redhat/singular.spec
+cp $BUILD_DIR/Singular/all.lib  singular-$VERSION/Singular/LIB/.
 tar cf singular-$TARVERSION.tar singular-$VERSION
 gzip -9 -f singular-$TARVERSION.tar
 command rm -rf singular-$VERSION
+
+else
+  echo BUILD_DIR is not set correctly, use
+  echo make dist
+  echo in a configured directory
+  exit 1
+fi
